@@ -27,6 +27,15 @@
 class GR_Graphics;
 class UT_ByteBuf;
 
+class ABI_EXPORT GR_Image_Point
+{
+public:
+  GR_Image_Point(void) : m_iX(0),m_iY(0)
+    {}
+  UT_sint32 m_iX;
+  UT_sint32 m_iY;
+};
+
 class ABI_EXPORT GR_Image
 {
 public:
@@ -47,7 +56,21 @@ public:
 	virtual void        scaleImageTo(GR_Graphics * pG, const UT_Rect & rec);
 	void				getName(char* szName) const;
 	void getName ( UT_String & copy ) const;
-								
+	/*!
+	 * Returns true if the image has any alpha in it.
+	 */
+	virtual bool hasAlpha(void) const = 0;
+	/*! 
+	 * Returns true if pixel at point (x,y) in device units is transparent.
+	 */
+	virtual bool isTransparentAt(UT_sint32 x, UT_sint32 y) = 0;
+	void         GenerateOutline(void);
+	void         DestroyOutline(void);
+	UT_sint32 GetOffsetFromLeft(GR_Graphics * pG, UT_sint32 pad, UT_sint32 yTop, UT_sint32 height);
+	UT_sint32 GetOffsetFromRight(GR_Graphics * pG, UT_sint32 pad, UT_sint32 yTop, UT_sint32 height);
+
+	bool isOutLinePresent(void) const
+	  { return (m_vecOutLine.getItemCount() > 0);}
    	enum GRType {
 	   GRT_Unknown,
 	   GRT_Raster,
@@ -68,11 +91,14 @@ private:
    	UT_String		m_szName;
 	UT_sint32			m_iDisplayWidth;
 	UT_sint32			m_iDisplayHeight;
+	UT_GenericVector<GR_Image_Point *> m_vecOutLine;
 };
 
 class ABI_EXPORT GR_RasterImage : public GR_Image
 {
 public:
+	virtual bool hasAlpha(void) const = 0;
+	virtual bool isTransparentAt(UT_sint32 x, UT_sint32 y) = 0;
    	virtual GRType		getType() const { return GRT_Raster; }
 };
 
