@@ -22,46 +22,33 @@
 
 #include <stdlib.h>
 
-typedef void* pair_type;
+/* pre-emptive dismissal; ut_types.h is needed by just about everything,
+ * so even if it's commented out in-file that's still a lot of work for
+ * the preprocessor to do...
+ */
+#ifndef UT_TYPES_H
+#include "ut_types.h"
+#endif
 
-#ifndef ABI_OPT_STL
 
-class UT_Pair
+template <class T, class U> class  ABI_EXPORT UT_Pair
 {
 public:
-	UT_Pair(const pair_type first, const pair_type second);
-	~UT_Pair();
+	UT_Pair(T first, U second)
+		:	m_first(first),
+			m_second(second) {}
+	virtual ~UT_Pair()
+		{ }
 
-	const pair_type& first() const  { return m_first; }
-	const pair_type& second() const { return m_second; }
-	pair_type& first()  { return m_first; }
-	pair_type& second() { return m_second; }
+	const T first() const  
+		{ return m_first; }
+	const U second() const 
+		{ return m_second; }
 
 private:
-	pair_type m_first;
-	pair_type m_second;
+	const T m_first;
+	const U m_second;
 };
 
-#else /* ABI_OPT_STL */
-
-#include <utility>
-
-class UT_Pair : public XAP_AbiObject
-{
-public:
-	UT_Pair(const pair_type first, const pair_type second);
-	~UT_Pair();
-
-	const pair_type& first()  const { m_pair.first; }
-	const pair_type& second() const { m_pair.second; }
-	pair_type& first()  { return m_pair.first; }
-	pair_type& second() { return m_pair.second; }
-
-private:
-	std::pair<pair_type, pair_type>	m_pair;
-};
-
-
-#endif /* ABI_OPT_STL */
 
 #endif
