@@ -22,10 +22,16 @@
 #ifndef UT_PNG_H
 #define UT_PNG_H
 
+/* pre-emptive dismissal; ut_types.h is needed by just about everything,
+ * so even if it's commented out in-file that's still a lot of work for
+ * the preprocessor to do...
+ */
+#ifndef UT_TYPES_H
 #include "ut_types.h"
+#endif
 
-class UT_ByteBuf;
+#include "ut_bytebuf.h"
 
-bool UT_PNG_getDimensions(const UT_ByteBuf* pBB, UT_sint32& iImageWidth, UT_sint32& iImageHeight);
+ABI_EXPORT bool UT_PNG_getDimensions(const UT_ByteBuf* pBB, UT_sint32& iImageWidth, UT_sint32& iImageHeight);
 
 #endif /* UT_PNG_H */
