@@ -145,6 +145,53 @@ bool XAP_UnixFontHandle::doesGlyphExist(UT_UCS4Char g)
 	return m_font->doesGlyphExist(g);
 }
 
+static float fixedToFloat(signed long in)
+{
+  float res = static_cast<float>(in*64);
+}
+//
+// UT_Rect of glyph in Logical units.
+// rec.left = bearing Left (distance from origin to start)
+// rec.width = widht of the glyph
+// rec.top = distance from the origin to the top of the glyph
+// rec.height = total height of the glyph
+//
+bool  XAP_UnixFontHandle::glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG) 
+{
+	UT_return_val_if_fail( pG, false );
+	
+	XftFaceLocker locker(m_font->getLayoutXftFont(GR_CharWidthsCache::CACHE_FONT_SIZE));
+
+	FT_Face pFace = locker.getFace();
+
+	UT_UCS4Char cChar = g;
+	
+	if(m_font->isSymbol())
+		cChar = static_cast<UT_UCS4Char>(adobeToUnicode(g));
+	
+	FT_UInt glyph_index = FT_Get_Char_Index(pFace, cChar);
+
+	FT_Error error = FT_Load_Glyph(pFace, glyph_index,
+								   FT_LOAD_LINEAR_DESIGN |
+								   FT_LOAD_IGNORE_TRANSFORM |
+								   FT_LOAD_NO_BITMAP |
+								   FT_LOAD_NO_SCALE);
+	if (error)
+	{
+		return false;
+	}
+
+	UT_uint32 iSize = m_size * pG->getResolution() / pG->getDeviceResolution();
+	
+	rec.left   = static_cast<UT_sint32>(fontPoints2float(iSize, pFace, pFace->glyph->metrics.horiBearingX));
+	rec.width  = static_cast<UT_sint32>(fontPoints2float(iSize, pFace, pFace->glyph->metrics.width));
+	rec.top    = static_cast<UT_sint32>(fontPoints2float(iSize, pFace, pFace->glyph->metrics.horiBearingY));
+	rec.height = static_cast<UT_sint32>(fontPoints2float(iSize, pFace, pFace->glyph->metrics.height));
+	
+	UT_DEBUGMSG(("GlyphBox: left %d width %d top %d height %d \n",rec.left,rec.width,rec.top,rec.height));
+
+	return true;
+}
 /*******************************************************************/
 
 XAP_UnixFont::XAP_UnixFont(XAP_UnixFontManager * pFM)
@@ -367,6 +414,24 @@ XAP_UnixFont::style XAP_UnixFont::getStyle(void) const
 	return m_style;
 }
 
+PangoStyle XAP_UnixFont::getPangoStyle(void) const
+{
+	PangoStyle ret = PANGO_STYLE_NORMAL;
+	if (m_style == STYLE_ITALIC || m_style == STYLE_BOLD_ITALIC) {
+		ret = PANGO_STYLE_ITALIC;
+	}
+	return ret;
+}
+
+PangoWeight XAP_UnixFont::getPangoWeight(void) const
+{
+	PangoWeight ret = PANGO_WEIGHT_NORMAL;
+	if (m_style == STYLE_BOLD || m_style == STYLE_BOLD_ITALIC || m_style == STYLE_BOLD_OUTLINE) {
+		ret = PANGO_WEIGHT_BOLD;
+	}
+	return ret;
+}
+
 const char *XAP_UnixFont::getFontfile(void) const
 {
 	return m_fontfile;
@@ -741,12 +806,13 @@ XftFont *XAP_UnixFont::getLayoutXftFont(UT_uint32 pixelsize) const
 // returns different fonts depending on the zoom percentage
 XftFont *XAP_UnixFont::getDeviceXftFont(UT_uint32 pixelsize, UT_uint32 zoomPercentage) const
 {
-	XftFont *pXftFont = getFontFromCache(pixelsize*zoomPercentage/100, false, zoomPercentage);
+#define extra_font_size 0.5 // was 1.5
+	XftFont *pXftFont = getFontFromCache(static_cast<UT_sint32>(extra_font_size + static_cast<double>(pixelsize*zoomPercentage)/static_cast<double>(100.)) , false, zoomPercentage);
 	if (pXftFont)
 		return pXftFont;
 
-	fetchXftFont(pixelsize*zoomPercentage/100);
-	return getFontFromCache(pixelsize*zoomPercentage/100, false, zoomPercentage);
+	fetchXftFont(static_cast<UT_sint32>(extra_font_size + static_cast<double>(pixelsize*zoomPercentage)/static_cast<double>(100.)));
+	return getFontFromCache(static_cast<UT_sint32>(extra_font_size + static_cast<double>(pixelsize*zoomPercentage)/static_cast<double>(100.)), false, zoomPercentage);
 }
 
 XftFont *XAP_UnixFont::getFontFromCache(UT_uint32 pixelsize, bool /*bIsLayout*/, UT_uint32 /*zoomPercentage*/) const
