@@ -30,7 +30,6 @@
 #include "gr_Win32Graphics.h"
 
 /*****************************************************************/
-/*
 static UINT CALLBACK s_PrintHookProc(
   HWND hdlg,      // handle to the dialog box window
   UINT uiMsg,     // message identifier
@@ -38,35 +37,78 @@ static UINT CALLBACK s_PrintHookProc(
   LPARAM lParam   // message parameter
   )
 {
-	// the only thing we want to do at the moment is to hide the
-	// selection radio if it is disabled
+	XAP_Win32Dialog_Print * pThis = NULL;
+	
 	if(uiMsg == WM_INITDIALOG)
 	{
 		PRINTDLG * pDlgInfo = (PRINTDLG*)lParam;
-#if 0
+		pThis = (XAP_Win32Dialog_Print*)pDlgInfo->lCustData;
+		SetWindowLong(hdlg, DWL_USER, (LONG) pThis);
+
+		// reset the 'closed' flag which indicates that the dialog should be considered
+		// 'closed' rather than aborted
+		pThis->setClosed(false);
+
+		// hide the selection radio since we do not support selection printing
 		if(pDlgInfo->Flags & PD_NOSELECTION)
 		{
 			HWND wPrintSelectionRadio = GetDlgItem(hdlg, rad2);
 			ShowWindow(wPrintSelectionRadio, SW_HIDE);
 		}
-#endif
-#if 0
-		// notify layout of currently selected printer
-		GR_Win32Graphics* pG=(GR_Win32Graphics*)XAP_App::getApp()->getLastFocussedFrame()->getCurrentView()->getGraphics();
-		UT_return_val_if_fail( pG, 0 );
-		pG->setPrintDC(pDlgInfo->hDC);
+
+		// remember the original printer selected; we use this to decide if upon closure
+		// we should notify associated views to rebuild to reflect the new font metrics
+		UT_uint32 iPrinter = SendDlgItemMessage(hdlg, cmb4, CB_GETCURSEL, 0, 0);
+		pThis->setOrigPrinter(iPrinter);
+		pThis->setNewPrinter(iPrinter);
 	}
 	else
 	{
-		UT_DEBUGMSG(("PrintDlg notification: 0x%x\n", uiMsg));
-#endif
+		pThis = (XAP_Win32Dialog_Print*)GetWindowLong(hdlg, DWL_USER);
 	}
 	
+	if(uiMsg == WM_COMMAND)
+	{
+		// if the command indicates that the user changed printer, we change the Cancel
+		// button to Close; make sure we have valid this pointer.
+		if((int)LOWORD(wParam) == cmb4 && HIWORD(wParam) == CBN_SELCHANGE)
+		{
+			pThis->setNewPrinter(SendDlgItemMessage(hdlg, cmb4, CB_GETCURSEL, 0, 0));
 
-	// we want to respond to changes to printer selection ...
+			XAP_App*              pApp        = XAP_App::getApp();
+			const XAP_StringSet*  pSS         = pApp->getStringSet();
+			
+			if(pThis->getNewPrinter() != pThis->getOrigPrinter())
+			{
+				SetDlgItemText(hdlg,IDCANCEL,pSS->getValue(XAP_STRING_ID_DLG_Close));
+			}
+			else
+			{
+				// the user set the printer back to what it used to be -- revert back to
+				// cancel button
+				SetDlgItemText(hdlg,IDCANCEL,pSS->getValue(XAP_STRING_ID_DLG_Cancel));
+			}
+		}
+		else if((int)LOWORD(wParam) == IDCANCEL && HIWORD(wParam) == 0)
+		{
+			// the user presed the Cancel/Close button; see if it is Cancel or Close
+			if(pThis->getNewPrinter() != pThis->getOrigPrinter())
+			{
+				// Different printer is selected to the one we started with, this is Close
+				// scenario -- eat this message and simulate click on the OK button
+				// instead so that the default dlg procedure fills the PRINTDLG struct
+				// with the data we need; set the m_bClosed flag, so we can differentiate
+				// this from the user pressing the OK button
+				pThis->setClosed(true);
+				PostMessage(hdlg, WM_COMMAND, IDOK, 0);
+
+				return 1; // eat the message
+			}
+		}
+	}
 	
 	return 0;
-}*/
+}
 
 XAP_Dialog * XAP_Win32Dialog_Print::static_constructor(XAP_DialogFactory * pFactory,
 													 XAP_Dialog_Id id)
@@ -76,8 +118,11 @@ XAP_Dialog * XAP_Win32Dialog_Print::static_constructor(XAP_DialogFactory * pFact
 }
 
 XAP_Win32Dialog_Print::XAP_Win32Dialog_Print(XAP_DialogFactory * pDlgFactory,
-										   XAP_Dialog_Id id)
-	: XAP_Dialog_Print(pDlgFactory,id)
+										   XAP_Dialog_Id id):
+	XAP_Dialog_Print(pDlgFactory,id),
+	m_iOrigPrinter(0),
+	m_iNewPrinter(0),
+	m_bClosed(false)
 {
 	m_pPersistPrintDlg = (PRINTDLG *)UT_calloc(1,sizeof(PRINTDLG));
 	UT_ASSERT(m_pPersistPrintDlg);
@@ -140,20 +185,19 @@ void XAP_Win32Dialog_Print::runModal(XAP_Frame * pFrame)
 
 	HWND hwnd = static_cast<XAP_Win32FrameImpl*>(pFrame->getFrameImpl())->getTopLevelWindow();
 
-	m_pPersistPrintDlg->hwndOwner		= hwnd;
-	m_pPersistPrintDlg->nFromPage		= (WORD)m_nFirstPage;
+	m_pPersistPrintDlg->hwndOwner	= hwnd;
+	m_pPersistPrintDlg->nFromPage	= (WORD)m_nFirstPage;
 	m_pPersistPrintDlg->nToPage		= (WORD)m_nLastPage;
-	m_pPersistPrintDlg->nMinPage		= (WORD)m_nFirstPage;
-	m_pPersistPrintDlg->nMaxPage		= (WORD)m_nLastPage;
-	m_pPersistPrintDlg->Flags		= PD_ALLPAGES | PD_RETURNDC /*| PD_ENABLEPRINTHOOK*/;
-	//m_pPersistPrintDlg->lpfnPrintHook   = s_PrintHookProc;
-	// we do not need this at the moment, but one day it will come handy in the hook procedure
+	m_pPersistPrintDlg->nMinPage	= (WORD)m_nFirstPage;
+	m_pPersistPrintDlg->nMaxPage	= (WORD)m_nLastPage;
+	m_pPersistPrintDlg->Flags		= PD_ALLPAGES | PD_RETURNDC | PD_ENABLEPRINTHOOK;
+	m_pPersistPrintDlg->lpfnPrintHook   = s_PrintHookProc;
 	m_pPersistPrintDlg->lCustData       = (DWORD)this;
 		
 	if (!m_bEnablePageRange)
-		m_pPersistPrintDlg->Flags		|= PD_NOPAGENUMS;
+		m_pPersistPrintDlg->Flags	|= PD_NOPAGENUMS;
 	if (!m_bEnablePrintSelection)
-		m_pPersistPrintDlg->Flags		|= PD_NOSELECTION;
+		m_pPersistPrintDlg->Flags	|= PD_NOSELECTION;
 	if (m_bEnablePrintToFile)
 	{
 		if (m_bDoPrintToFile)
@@ -161,7 +205,7 @@ void XAP_Win32Dialog_Print::runModal(XAP_Frame * pFrame)
 	}
 	else
 	{
-		m_pPersistPrintDlg->Flags		|= PD_HIDEPRINTTOFILE;
+		m_pPersistPrintDlg->Flags	|= PD_HIDEPRINTTOFILE;
 	}
 	
 	if (!m_bPersistValid)				// first time
@@ -198,15 +242,37 @@ void XAP_Win32Dialog_Print::runModal(XAP_Frame * pFrame)
 			
 			GlobalUnlock(m_pPersistPrintDlg->hDevMode);
 			GlobalUnlock(m_pPersistPrintDlg->hDevNames);
+
+			// notify layout of printer (this results in redoing doc layout !!!)
+			GR_Win32Graphics* pG=(GR_Win32Graphics*)XAP_App::getApp()->getLastFocussedFrame()->getCurrentView()->getGraphics();
+			UT_return_if_fail( pG );
+			pG->setPrintDC(m_pPersistPrintDlg->hDC);
 		}
 	}
 	else if (PrintDlg(m_pPersistPrintDlg))		// raise the actual dialog.
 	{
 		_extractResults(pFrame);
+
+		// notify layout of printer (this results in redoing doc layout !!!)
+		GR_Win32Graphics* pG=(GR_Win32Graphics*)XAP_App::getApp()->getLastFocussedFrame()->getCurrentView()->getGraphics();
+		UT_return_if_fail( pG );
+		pG->setPrintDC(m_pPersistPrintDlg->hDC);
+
+		// we need to differentiate between actual OK being pressed by user and simulated
+		// by our hook procedure -- in the later case we want to return a_CANCEL so that
+		// the document does not get sent to the printer. We will also set m_bPersistValid
+		// as if OK was clicked
+		if(m_bClosed)
+		{
+			m_answer = a_CANCEL;
+			m_bPersistValid = true;
+		}
+		
 	}
 	else
 	{
-		UT_DEBUGMSG(("Printer dialog failed: reason=0x%x\n", CommDlgExtendedError()));
+		// The user clicked the cancel button and did not change the printer, there is
+		// nothing for us to do
 		m_answer = a_CANCEL;
 	}
 
@@ -230,6 +296,7 @@ void XAP_Win32Dialog_Print::_extractResults(XAP_Frame *pFrame)
 	m_bCollate	= ((pDevMode->dmCollate  & DMCOLLATE_TRUE) != 0);		
 	pDevMode->dmCopies = 1;
 	pDevMode->dmCollate = DMCOLLATE_FALSE;
+	pDevMode->dmFields = DM_COPIES | DM_COLLATE;
 	GlobalUnlock(m_pPersistPrintDlg->hDevMode);
 
 	// any changes to the DEVMODE structure must be followed by this call before the
@@ -256,12 +323,6 @@ void XAP_Win32Dialog_Print::_extractResults(XAP_Frame *pFrame)
 	}
 
 	m_answer = a_OK;
-	{
-		// notify layout of printer (this results in redoing doc layout !!!)
-		GR_Win32Graphics* pG=(GR_Win32Graphics*)XAP_App::getApp()->getLastFocussedFrame()->getCurrentView()->getGraphics();
-		UT_return_if_fail( pG );
-		pG->setPrintDC(m_pPersistPrintDlg->hDC);
-	}
 	return;
 
 Fail:
