@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiSource Application Framework
  * Copyright (C) 2001 Dom Lachowicz <cinamod@hotmail.com>
  * 
@@ -25,51 +27,48 @@
 // Pass the intended file name (dll, so, whatever)
 // To a valid child instance of this class.
 
+#include <stdio.h>
+
+/* pre-emptive dismissal; ut_types.h is needed by just about everything,
+ * so even if it's commented out in-file that's still a lot of work for
+ * the preprocessor to do...
+ */
+#ifndef UT_TYPES_H
 #include "ut_types.h"
-
-// TODO: make this turn into extern/_declspec(dllexport) depending on platform
-#ifdef WIN32
-
-#include <windows.h>
-#define WIN32_LEAN_AND_MEAN
-
-#define ABI_FAR __declspec( dllexport )
-#define ABI_CALL extern "C"
-#define ABI_FAR_CALL ABI_CALL ABI_FAR
-
-#define ABI_PLUGIN_DECLARE(name) BOOL APIENTRY DllMain( HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved ) { return TRUE; } 
-
-#else
-
-#define ABI_FAR
-#define ABI_CALL extern "C"
-#define ABI_FAR_CALL ABI_CALL ABI_FAR
-#define ABI_PLUGIN_DECLARE(name)
-
 #endif
 
-
+#ifdef WIN32
+  #define WIN32_LEAN_AND_MEAN
+  #include <windows.h>
+  #define ABI_PLUGIN_DECLARE(name) static HANDLE s_hModule = static_cast<HANDLE>(NULL); BOOL APIENTRY DllMain( HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved ) { s_hModule = hModule; return TRUE; } 
+#else
+  #define ABI_PLUGIN_DECLARE(name)
+#endif
 
 // we want to have C linkage for both 
 // this and for all of our required functions
 extern "C" {
-	typedef struct {
+	typedef ABI_EXPORT struct XAP_ModuleInfo {
 		char * name;
 		char * desc;
 		char * version;
 		char * author;
 		char * usage;
-	} XAP_ModuleInfo;
+	};
 }
 
 class XAP_ModuleManager;
 
-class XAP_Module {
+typedef int (*XAP_Plugin_Registration) (XAP_ModuleInfo * mi);
+typedef int (*XAP_Plugin_VersionCheck) (UT_uint32 major, UT_uint32 minor, UT_uint32 release);
+
+class ABI_EXPORT XAP_Module {
 
 	friend class XAP_ModuleManager;
 
 protected:
 	XAP_Module ();
+
 	virtual ~XAP_Module (void);
 	
 	// load this module into memory. true on success
@@ -78,8 +77,15 @@ protected:
 	// unload this module from memory. true on success
 	virtual bool unload (void) = 0;	
 	
-private:
+public:
+	// marks the module as loaded; returns false if module is already loaded
+	bool setSymbols (XAP_Plugin_Registration fnRegister,
+					 XAP_Plugin_Registration fnDeregister,
+					 XAP_Plugin_VersionCheck fnSupportsVersion);
 
+	bool registered ();
+private:
+	bool registerPending ();
 	// silly names. fscking c/c++ has the keyword 'register' taken #:^)
 	bool registerThySelf ();
 	bool unregisterThySelf ();
@@ -94,7 +100,7 @@ public:
 	// *symbol refers to the actual representation of @symbol_name
 	//
 	// void (*func) (XAP_ModuleInfo *);
-	// resolveSymbol ("abi_plugin_init", &(void *)func);
+	// resolveSymbol ("abi_plugin_init", static_cast<void **>(&func));
 	// int result = func (&m_info);
 	virtual bool resolveSymbol (const char * symbol_name, void ** symbol) = 0;
 	
@@ -109,16 +115,50 @@ public:
 	inline XAP_ModuleManager * getCreator () const {return m_creator;}
 
 	inline const XAP_ModuleInfo *getModuleInfo (void) const {return &m_info;}
-	
+
 private:
 
+	XAP_Plugin_Registration		m_fnRegister;
+	XAP_Plugin_Registration		m_fnDeregister;
+	XAP_Plugin_VersionCheck		m_fnSupportsVersion;
+
 	XAP_ModuleManager   * m_creator;
 	bool                  m_bLoaded;
 	bool                  m_bRegistered;
+	int                   m_iStatus;
+	const char          * m_szSPI;
 	XAP_ModuleInfo        m_info;
 };
 
-#define ABI_VERSION_STRING "0.9.4"
-#define isCurrentAbiVersion(a,b,c) (((a)==0) && ((b)==9) && ((c)==4))
+#if 0 // TODO: make me 100% and turn me on again one day
+#ifndef ABI_PLUGIN_SOURCE
+static inline bool
+_isCurrentAbiVersion(int a, int b, int c)
+{
+       int _a, _b, _c, _n;
+
+       _n = sscanf( ABI_BUILD_VERSION , "%d.%d.%d", &_a, &_b, &_c);
+
+       if (_n <= 1 || _a != a || _b != b)
+               return false;
+
+       if (3 == _n && _c != c)
+               return false;
+
+       return true;
+}
+#endif /* ! ABI_PLUGIN_SOURCE */
+
+#define isCurrentAbiVersion(a,b,c) _isCurrentAbiVersion(a, b, c)
+
+#else
+#define isCurrentAbiVersion(a,b,c) true
+#endif
+
+#ifdef ABI_PLUGIN_SOURCE
+#define ABI_VERSION_STRING ABI_PLUGIN_VERSION
+#else
+#define ABI_VERSION_STRING ABI_BUILD_VERSION
+#endif
 
 #endif /* XAP_MODULE_H */
