@@ -25,8 +25,8 @@
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
 
-#include "xap_Spider.h"
 #include "xap_ModuleManager.h"
+#include "xap_Prefs.h"
 #include "ut_string_class.h"
 #include "ut_path.h"
 
@@ -36,10 +36,6 @@
   #include "xap_Win32Module.h"
   #define MODULE_CLASS XAP_Win32Module
 
-#elif defined (__BEOS__)
-  #include "xap_BeOSModule.h"
-  #define MODULE_CLASS XAP_BeOSModule
-
 #elif defined (__QNXNTO__)
   #include "xap_QNXModule.h"
   #define MODULE_CLASS XAP_QNXModule
@@ -67,11 +63,23 @@
 
 #endif
 
+// log information about plugin loading into the <log> section of AbiWord.profile
+// (we save the prefs file after each call, so as to maximise the information we have in
+// case a plugin crashes and the crash handler does not get a chance to save it for us
+#define XAP_MODULE_MANAGER_LOAD_LOG(msg1, msg2)                                      \
+if(XAP_App::getApp() && XAP_App::getApp()->getPrefs())                               \
+{                                                                                    \
+    UT_String __s;                                                                   \
+    UT_String_sprintf(__s, "(L%d): %s %s", __LINE__, msg1, msg2);                    \
+	UT_DEBUGMSG(("%s",__s.c_str()));                                                 \
+    XAP_App::getApp()->getPrefs()->log("XAP_ModuleManager::loadModule", __s.c_str());\
+	XAP_App::getApp()->getPrefs()->savePrefsFile();                                  \
+}
+
 /*!
  * Protected destructor creates an instance of this module class
  */
-XAP_ModuleManager::XAP_ModuleManager ()	:
-  m_spider(new XAP_Spider)
+XAP_ModuleManager::XAP_ModuleManager ()
 {
 	m_modules = new UT_GenericVector<XAP_Module*> (11);
 }
@@ -81,7 +89,6 @@ XAP_ModuleManager::XAP_ModuleManager ()	:
  */
 XAP_ModuleManager::~XAP_ModuleManager ()
 {
-	DELETEP (m_spider);
 	UT_VECTOR_PURGEALL (MODULE_CLASS *, (*m_modules));
 	delete m_modules;
 }
@@ -100,10 +107,7 @@ XAP_ModuleManager & XAP_ModuleManager::instance ()
  *
  * \param szFilename - the .dll or .so on your system that you wish to load
  *
- * Note that if the plugin is a SPI then registration will be postponed until
- * XAP_ModuleManager::registerPending () is called.
- *
- * \return true if loaded (and, if not a SPI, registered) successfully, false otherwise
+ * \return true if loaded successfully, false otherwise
  */
 bool XAP_ModuleManager::loadModule (const char * szFilename)
 {
@@ -112,7 +116,10 @@ bool XAP_ModuleManager::loadModule (const char * szFilename)
 	if ( szFilename == 0) return false;
 	if (*szFilename == 0) return false;
 
+	XAP_MODULE_MANAGER_LOAD_LOG("loading", szFilename)
+
 	// check to see if plugin is already loaded
+	
 	XAP_Module* pModuleLoop = 0;
 	const UT_GenericVector<class XAP_Module *> *pVec = enumModules();
 	
@@ -148,11 +155,13 @@ bool XAP_ModuleManager::loadModule (const char * szFilename)
 	if (!pModule->load (szFilename))
 	{		
 		UT_DEBUGMSG (("Failed to load module %s\n", szFilename));
+		XAP_MODULE_MANAGER_LOAD_LOG("failed to load", szFilename)
 		
 		char * errorMsg = 0;
 		if (pModule->getErrorMsg (&errorMsg))
 		{	
 			UT_DEBUGMSG (("Reason: %s\n", errorMsg));
+			XAP_MODULE_MANAGER_LOAD_LOG("error msg", errorMsg)
 			FREEP (errorMsg);
 		}
 		delete pModule;
@@ -161,18 +170,19 @@ bool XAP_ModuleManager::loadModule (const char * szFilename)
 
 	/* assign the module's creator to be us, etc.
 	 */
-	pModule->setSpider (m_spider);
 	pModule->setLoaded (true);
 	pModule->setCreator (this);
 
 	if (!pModule->registerThySelf ())
 	{
 		UT_DEBUGMSG (("Failed to register module %s\n", szFilename));
+		XAP_MODULE_MANAGER_LOAD_LOG("failed to register", szFilename)
 		
 		char * errorMsg = 0;
 		if (pModule->getErrorMsg (&errorMsg))
 		{	
 			UT_DEBUGMSG (("Reason: %s\n", errorMsg));
+			XAP_MODULE_MANAGER_LOAD_LOG("error msg", errorMsg)
 			FREEP (errorMsg);
 		}
 		pModule->unload ();
@@ -181,6 +191,7 @@ bool XAP_ModuleManager::loadModule (const char * szFilename)
 	}
 	if (m_modules->addItem (pModule)) // an error occurred...
 	{
+		XAP_MODULE_MANAGER_LOAD_LOG("could not add", szFilename)
 		pModule->unregisterThySelf ();
 		pModule->unload ();
 		delete pModule;
@@ -189,9 +200,13 @@ bool XAP_ModuleManager::loadModule (const char * szFilename)
 
 	/* we (somehow :^) got here. count our blessings and return
 	 */
+	XAP_MODULE_MANAGER_LOAD_LOG("success", szFilename)
 	return true;
 }
 
+/*!
+* Load builtin plugins.
+*/
 bool XAP_ModuleManager::loadPreloaded (XAP_Plugin_Registration fnRegister,
 									   XAP_Plugin_Registration fnDeregister,
 									   XAP_Plugin_VersionCheck fnSupportsVersion)
@@ -219,7 +234,6 @@ bool XAP_ModuleManager::loadPreloaded (XAP_Plugin_Registration fnRegister,
 
 	/* assign the module's creator to be us, etc.
 	 */
-	pModule->setSpider (m_spider);
 	pModule->setLoaded (true);
 	pModule->setCreator (this);
 
@@ -242,32 +256,6 @@ bool XAP_ModuleManager::loadPreloaded (XAP_Plugin_Registration fnRegister,
 	return true;
 }
 
-/*!
- * Register any SPI plugins/modules pending registration.
- * 
- * This procedure calls XAP_ModuleManager::unloadUnregistered
- * 
- * \return number of plugins registered
- */
-UT_uint32 XAP_ModuleManager::registerPending ()
-{
-	UT_uint32 count = 0;
-
-	if (m_spider) count = m_spider->register_spies ();
-
-	/* The Spider does the plugin registration (see above)
-	 * but the module info has to be updated:
-	 */
-	for (UT_uint32 i = 0; i < m_modules->size (); i++)
-	{
-		XAP_Module * pModule = reinterpret_cast<XAP_Module *>(m_modules->getNthItem(i));
-		if (pModule) pModule->registerPending ();
-	}
-	unloadUnregistered ();
-
-	return count;
-}
-
 /*!
  * Unload a module
  *
@@ -276,7 +264,7 @@ UT_uint32 XAP_ModuleManager::registerPending ()
  * WARNING: zero or more plugins/modules may be unloaded as a result of
  * calling XAP_ModuleManager::unloadModule since the plugin may be unloaded
  * already or other plugins may depend on this one and be unloaded
- * automatically (this procedure calls XAP_ModuleManager::unloadUnregistered).
+ * automatically.
  * 
  * Don't assume anything, therefore, about the UT_Vector returned by
  * XAP_ModuleManager::enumModules
@@ -297,10 +285,9 @@ void XAP_ModuleManager::unloadModule (XAP_Module * pModule)
 		return;
 	}
 	unloadModule (ndx);
-	unloadUnregistered ();
 }
 
-/* for use by unloadAllPlugins, unloadModule & unloadUnregistered only!
+/* for use by unloadAllPlugins, unloadModule only!
  */
 void XAP_ModuleManager::unloadModule (UT_sint32 ndx)
 {
@@ -327,31 +314,6 @@ void XAP_ModuleManager::unloadModule (UT_sint32 ndx)
 	delete pModule;
 }
 
-/*!
- * Unload any SPI plugins/modules that haven't been registered
- */
-void XAP_ModuleManager::unloadUnregistered ()
-{
-	UT_return_if_fail(m_spider  != NULL);
-	UT_return_if_fail(m_modules != NULL);
-
-	UT_sint32 bad_module;
-	do
-	{
-		bad_module = -1;
-		for (UT_uint32 i = 0; i < m_modules->size (); i++)
-		{
-			XAP_Module * pModule = m_modules->getNthItem (i);
-			if (!pModule->registered ())
-			{
-				bad_module = static_cast<UT_sint32>(i);
-				break;
-			}
-		}
-		if (bad_module != -1) unloadModule (bad_module);
-	} while (bad_module != -1);
-}
-
 /*!
  * Enumerate the modules loaded thus far
  *
