@@ -36,7 +36,6 @@
 //###TF#include "px_CR_Bookmark.h"
 #include "px_CR_FmtMarkChange.h"
 #include "px_CR_Glob.h"
-#include "fv_View.h"
 #include "fl_DocListener.h"
 #include "fl_DocLayout.h"
 #include "fl_SectionLayout.h"
@@ -44,12 +43,15 @@
 #include "fl_ContainerLayout.h"
 #include "fl_FootnoteLayout.h"
 #include "fl_TableLayout.h"
+#include "fl_TOCLayout.h"
 #include "fp_Line.h"
 #include "fp_Run.h"
 #include "gr_Graphics.h"
 #include "pd_Document.h"
 #include "pp_AttrProp.h"
 #include "xap_Frame.h"
+#include "xap_Prefs.h"
+#include "fv_View.h"
 
 #define UPDATE_LAYOUT_ON_SIGNAL
 
@@ -80,6 +82,8 @@ fl_DocListener::fl_DocListener(PD_Document* doc, FL_DocLayout *pLayout)
 	m_sLastContainerLayout.push(NULL);
 	m_bFootnoteInProgress = false;
 	m_bEndFootnoteProcessedInBlock = false;
+	m_bCacheChanges = false;
+	m_chgMaskCached = AV_CHG_NONE;
 }
 
 /*!
@@ -166,6 +170,31 @@ bool fl_DocListener::populate(PL_StruxFmtHandle sfh,
 		}
 		else
 			bResult = pCLSL->bl_doclistener_populateObject(pCL, blockOffset,pcro);
+
+#if 0
+		// if the inserted object is a bookmark, we have to notify TOCs of bookmark change in case
+		// any of them is restricted by the given bookmark
+		if(pcro->getObjectType() == PTO_Bookmark)
+		{
+			if(pL->getType() == PTX_Block)
+			{
+				fl_BlockLayout * pBL = static_cast<fl_BlockLayout *>(pL);
+				fp_Run * pRun = pBL->findRunAtOffset(pcro->getBlockOffset());
+				if(pRun && pRun->getType() == FPRUN_BOOKMARK)
+				{
+					fp_BookmarkRun * pB = static_cast<fp_BookmarkRun*>(pRun);
+
+					// only do this when the end-object is inserted ...
+					if(!pB->isStartOfBookmark())
+						m_pLayout->updateTOCsOnBookmarkChange(pB->getName());
+				}
+				else if(pRun && pRun->getType() != FPRUN_BOOKMARK)
+				{
+					UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+				}
+			}
+		}
+#endif
 		goto finish_up;
 	}
 
@@ -178,6 +207,7 @@ bool fl_DocListener::populate(PL_StruxFmtHandle sfh,
 		{
 			m_pDoc->miniDump(pL->getStruxDocHandle(),8);
 			UT_DEBUGMSG(("Illegal strux is %x \n",pL->getStruxDocHandle()));
+			UT_return_val_if_fail((pL->getType() == PTX_Block),false);
 		}			
 		UT_ASSERT(pL->getType() == PTX_Block);
 		fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pL);
@@ -228,6 +258,10 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 			{
 				xxx_UT_DEBUGMSG(("Null Update in Populate Strux \n"));
 				pFrame->nullUpdate();
+				PT_DocPosition pos = pcrx->getPosition();
+				UT_uint32 percentFilled = 100*pos/m_pLayout->getDocSize();
+				m_pLayout->setPercentFilled(percentFilled);
+				UT_DEBUGMSG(("Percent filled = %d \n",percentFilled));
 				if(countStrux > 60)
 				{
 					if(countStrux < 300)
@@ -430,6 +464,33 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 		UT_ASSERT(m_pCurrentSL->getContainerType() == FL_CONTAINER_DOCSECTION);
 		break;
 	}
+	case PTX_SectionTOC:
+	{
+		UT_ASSERT(m_pCurrentSL);
+	    fl_SectionLayout * pSL = NULL;
+		UT_DEBUGMSG(("fl_DocListener::populateStrux for 'SectionTOC'\n"));
+		pSL = (fl_SectionLayout *) m_pCurrentSL->append(sdh, pcr->getIndexAP(),FL_CONTAINER_TOC);
+		*psfh = (PL_StruxFmtHandle)pSL;
+		m_pCurrentSL = (fl_SectionLayout*)pSL;
+		break;
+	}
+
+	case PTX_EndTOC:
+	{
+//
+// CurrentSL is a TOC. Return this and set the m_pCurrentSL to it's 
+// container
+//
+		fl_ContainerLayout * pCL = m_pCurrentSL;
+#if DEBUG
+		UT_DEBUGMSG(("fl_DocListener::populateStrux for 'EndTOC'\n"));
+		UT_ASSERT(pCL->getContainerType() == FL_CONTAINER_TOC);
+#endif
+		*psfh = (PL_StruxFmtHandle) pCL;
+		static_cast<fl_TOCLayout *>(pCL)->setTOCEndIn();
+		m_pCurrentSL = (fl_SectionLayout *) static_cast<fl_TOCLayout *>(m_pCurrentSL)->getDocSectionLayout();
+		break;
+	}
 	case PTX_SectionHdrFtr:
 		// This path is taken on a change of page type. Eg A4 => letter.
 	{
@@ -490,6 +551,11 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 	}
 	case PTX_Block:
 	{
+		UT_DEBUGMSG(("SEVIOR: Doing populate block sdh %x \n",sdh));
+		if(m_pCurrentSL == NULL)
+		{
+			m_pDoc->miniDump(sdh,6);
+		}
 		UT_ASSERT(m_pCurrentSL);
 //
 // Look if we're inside a table. If so append this block to a cell.
@@ -525,11 +591,16 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 				{
 					reason = (UT_uint32) FL_DocLayout::bgcrSpelling;
 				}
-				m_pLayout->queueBlockForBackgroundCheck(reason, (fl_BlockLayout *)pCL,true);
+				if( m_pLayout->getAutoGrammarCheck())
+				{
+					reason = reason | (UT_uint32) FL_DocLayout::bgcrGrammar;
+				}
+				m_pLayout->queueBlockForBackgroundCheck(reason, (fl_BlockLayout *)pCL,false);
 			}
 		}
 		else
 		{
+			UT_return_val_if_fail( m_pCurrentSL,false );
 			pCL = m_pCurrentSL->append(sdh, pcr->getIndexAP(),FL_CONTAINER_BLOCK);
 			if (!pCL)
 			{
@@ -545,7 +616,11 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 				{
 					reason = (UT_uint32) FL_DocLayout::bgcrSpelling;
 				}
-				m_pLayout->queueBlockForBackgroundCheck(reason, (fl_BlockLayout *)pCL,true);
+				if( m_pLayout->getAutoGrammarCheck())
+				{
+					reason = reason | (UT_uint32) FL_DocLayout::bgcrGrammar;
+				}
+				m_pLayout->queueBlockForBackgroundCheck(reason, (fl_BlockLayout *)pCL,false);
 			}
 		}
 
@@ -567,10 +642,13 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 		UT_ASSERT(m_pCurrentSL);
 		
 		// Append a new TableLayout to the SectionLayout or CellLayout
-		
 		fl_ContainerLayout * pCon = getTopContainerLayout();
 		fl_ContainerLayout*	pCL = NULL;
 		UT_DEBUGMSG(("!!!!Appending Table \n"));
+		if(m_pCurrentSL->getHdrFtrLayout())
+		{
+			UT_DEBUGMSG(("Appending Table into HdrFtr %x \n",m_pCurrentSL->getHdrFtrLayout()));
+		}
 		if(pCon == NULL)
 		{
 			pCL = m_pCurrentSL->append(sdh, pcr->getIndexAP(),FL_CONTAINER_TABLE);
@@ -593,6 +671,63 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 		m_pDoc->setDontImmediatelyLayout(true);
 	}
 	break;
+	case PTX_SectionFrame:
+	{
+		UT_ASSERT(m_pCurrentSL);
+		UT_ASSERT(m_pCurrentSL->getContainerType() == FL_CONTAINER_DOCSECTION);
+		fl_ContainerLayout*	pCL = NULL;
+//
+// Look to see if we're in a table.
+//
+		fl_ContainerLayout * pCon = getTopContainerLayout();
+		if((pCon != NULL))
+		{
+			if(pCon->getContainerType() == FL_CONTAINER_CELL)
+			{
+				fl_ContainerLayout * pCL = pCon->append(sdh,pcr->getIndexAP(),FL_CONTAINER_FRAME);
+				m_pCurrentSL = static_cast<fl_SectionLayout *>(pCL);
+				*psfh = (PL_StruxFmtHandle)pCL;
+				break;
+			}
+#ifdef DEBUG
+			m_pDoc->miniDump(sdh,6);
+#endif
+			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		}
+
+		// Append a new FrameLayout to the SectionLayout
+
+
+		UT_DEBUGMSG(("!!!!Appending Frame \n"));
+		pCL = m_pCurrentSL->append(sdh, pcr->getIndexAP(),FL_CONTAINER_FRAME);
+		if (!pCL)
+		{
+			UT_DEBUGMSG(("no memory for TableLayout"));
+			return false;
+		}
+		m_pCurrentSL = static_cast<fl_SectionLayout *>(pCL);
+		*psfh = (PL_StruxFmtHandle)pCL;
+	}
+	break;
+	case PTX_EndFrame:
+	{
+		UT_ASSERT(m_pCurrentSL);
+		UT_ASSERT(m_pCurrentSL->getContainerType() == FL_CONTAINER_FRAME);
+		// Append a new FrameLayout to the SectionLayout
+
+		fl_ContainerLayout*	pCL = NULL;
+		UT_DEBUGMSG(("!!!!Appending EndFrame \n"));
+		pCL = m_pCurrentSL;
+		*psfh = (PL_StruxFmtHandle)pCL;
+		m_pCurrentSL = static_cast<fl_SectionLayout *>(pCL->myContainingLayout());
+		if(m_pCurrentSL->getContainerType() == FL_CONTAINER_CELL)
+		{
+			m_pCurrentSL = static_cast<fl_CellLayout *>(m_pCurrentSL)->getDocSectionLayout();
+		}
+		UT_ASSERT(m_pCurrentSL->getContainerType() == FL_CONTAINER_DOCSECTION);
+	}
+	break;
+
 	case PTX_SectionCell:
 	{
 		UT_ASSERT(m_pCurrentSL);
@@ -609,6 +744,10 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 			return false;
 		}
 		fl_TableLayout * pTable = (fl_TableLayout *) pCon;
+		if(pTable->getHdrFtrLayout())
+		{
+			UT_DEBUGMSG(("Appending a Cell to a Table in a HDrFtr %x \n",pTable->getHdrFtrLayout()));
+		}
 		fl_ContainerLayout*	pCL = pTable->append(sdh, pcr->getIndexAP(),FL_CONTAINER_CELL);
 		UT_DEBUGMSG(("SEVIOR: Appending Cell: layout is %x \n",pCL));
 		pTable->attachCell(pCL);
@@ -638,6 +777,7 @@ bool fl_DocListener::populateStrux(PL_StruxDocHandle sdh,
 		fl_TableLayout * pTL = static_cast<fl_TableLayout *>(pCon);
 		UT_DEBUGMSG(("SEVIOR: End table in doclistener \n"));
 		pTL->setDirty();
+		pTL->setEndTableIn();
 		pCon = getTopContainerLayout();
 		if(pCon == NULL)
 		{
@@ -727,6 +867,8 @@ fl_ContainerLayout * fl_DocListener::popContainerLayout(void)
 bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 							const PX_ChangeRecord * pcr)
 {
+	UT_return_val_if_fail( sfh, false );
+	
 	//UT_DEBUGMSG(("fl_DocListener::change\n"));
 	bool bResult = false;
 	AV_ChangeMask chgMask = AV_CHG_NONE;
@@ -767,6 +909,10 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 		const PX_ChangeRecord_Span * pcrs = static_cast<const PX_ChangeRecord_Span *> (pcr);
 
 		fl_Layout * pL = (fl_Layout *)sfh;
+		if(pL->getType() != PTX_Block)
+		{
+			m_pDoc->miniDump(pL->getStruxDocHandle(),6);
+		}
 		UT_ASSERT(pL->getType() == PTX_Block);
 		fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pL);
 		fl_SectionLayout* pCLSL = pCL->getSectionLayout();
@@ -842,6 +988,14 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 	{
 		const PX_ChangeRecord_FmtMark * pcrfm = static_cast<const PX_ChangeRecord_FmtMark *>(pcr);
 
+		if(!sfh)
+		{
+			// sometimes happens with revisions, not sure if this is a real problem, but
+			// assert anyway
+			UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+			goto finish_up;
+		}
+		
 		fl_Layout * pL = (fl_Layout *)sfh;
 		UT_ASSERT(pL->getType() == PTX_Block);
 		fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pL);
@@ -915,6 +1069,14 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 			m_pLayout->updateLayout();
 			goto finish_up;
 		}
+		case PTX_SectionTOC:
+		{
+			fl_Layout * pL = (fl_Layout *)sfh;
+			UT_ASSERT(pL->getType() == PTX_SectionTOC);
+			fl_TOCLayout * pFL = static_cast<fl_TOCLayout *>(pL);
+			pFL->doclistener_deleteStrux(pcrx);
+			goto finish_up;
+		}
 		case PTX_SectionFootnote:
 		{
 			fl_Layout * pL = (fl_Layout *)sfh;
@@ -947,6 +1109,18 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 			pCellL->doclistener_deleteStrux(pcrx);
 			goto finish_up;
 		}
+		case PTX_SectionFrame:
+		{
+			fl_Layout * pL = (fl_Layout *)sfh;
+			UT_ASSERT(pL->getType() == PTX_SectionFrame);
+			fl_FrameLayout * pFrameL = (fl_FrameLayout *) pL;
+			pFrameL->doclistener_deleteStrux(pcrx);
+			goto finish_up;
+		}
+		case PTX_EndFrame:
+		{
+			goto finish_up;
+		}
 		case PTX_EndTable:
 		{
 			goto finish_up;
@@ -955,6 +1129,10 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 		{
 			goto finish_up;
 		}
+		case PTX_EndTOC:
+		{
+			goto finish_up;
+		}
 		case PTX_EndFootnote:
 		{
 			fl_Layout * pL = (fl_Layout *)sfh;
@@ -996,6 +1174,7 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 		{
 		case PTX_Section:
 		case PTX_SectionEndnote:
+		case PTX_SectionFootnote:
 		{
 			fl_DocSectionLayout* pSL = static_cast<fl_DocSectionLayout*>(pL);
 			
@@ -1090,6 +1269,19 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 		}
 		case PTX_SectionHdrFtr:
 		{
+			if(pcrxc->isRevisionDelete())
+			{
+				fl_Layout * pL = (fl_Layout *)sfh;
+				UT_ASSERT(pL->getType() == PTX_SectionHdrFtr);
+				fl_HdrFtrSectionLayout * pSL = static_cast<fl_HdrFtrSectionLayout *>(pL);
+				//
+				// Nuke the HdrFtrSectionLayout and the shadows associated with it.
+				//
+				pSL->doclistener_deleteStrux(pcr); 
+				m_pLayout->updateLayout();
+				goto finish_up;
+			}
+			
 //
 // OK A previous "insertStrux" has created a HdrFtrSectionLayout but it
 // Doesn't know if it's a header or a footer or the DocSection and hences pages
@@ -1162,6 +1354,20 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 			bResult = pCL->doclistener_changeStrux(pcrxc);
 			goto finish_up;
 		}
+		case PTX_SectionFrame:
+		{
+			fl_FrameLayout * pFL = (fl_FrameLayout *) pL;
+			UT_ASSERT(pFL->getContainerType() == FL_CONTAINER_FRAME);
+			bResult = pFL->doclistener_changeStrux(pcrxc);
+			goto finish_up;
+		}
+		case PTX_SectionTOC:
+		{
+			fl_TOCLayout * pTOCL = (fl_TOCLayout *) pL;
+			UT_ASSERT(pTOCL->getContainerType() == FL_CONTAINER_TOC);
+			bResult = pTOCL->doclistener_changeStrux(pcrxc);
+			goto finish_up;
+		}
 		default:
 		{
 			UT_ASSERT(0);
@@ -1192,6 +1398,30 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 		}
 		else
 			bResult = pCLSL->bl_doclistener_insertObject(pCL, pcro);
+
+		// if the inserted object is a bookmark, we have to notify TOCs of bookmark change in case
+		// any of them is restricted by the given bookmark
+		if(pcro->getObjectType() == PTO_Bookmark)
+		{
+			if(pL->getType() == PTX_Block)
+			{
+				fl_BlockLayout * pBL = static_cast<fl_BlockLayout *>(pL);
+				fp_Run * pRun = pBL->findRunAtOffset(pcro->getBlockOffset());
+				if(pRun && pRun->getType() == FPRUN_BOOKMARK)
+				{
+					fp_BookmarkRun * pB = static_cast<fp_BookmarkRun*>(pRun);
+
+					// only do this when the end-object is inserted ...
+					if(!pB->isStartOfBookmark())
+						m_pLayout->updateTOCsOnBookmarkChange(pB->getName());
+				}
+				else if(pRun && pRun->getType() != FPRUN_BOOKMARK)
+				{
+					UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+				}
+			}
+		}
+		
 		goto finish_up;
 	}
 	case PX_ChangeRecord::PXT_DeleteObject:
@@ -1200,6 +1430,37 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 
 		fl_Layout * pL = (fl_Layout *)sfh;
 		UT_ASSERT(pL->getType() == PTX_Block);
+
+		// if the deleted object is a bookmark, we have to notify TOCs of bookmark change in case
+		// any of them is restricted by the given bookmark;
+		//
+		// this has to be done in two steps: before the layout is updated, we have to find out the
+		// name of the bookmark; after the layout is updated, we need to notify the TOCs.
+		UT_UTF8String sBookmark;
+		
+		if(pcro->getObjectType() == PTO_Bookmark)
+		{
+			if(pL->getType() == PTX_Block)
+			{
+				fl_BlockLayout * pBL = static_cast<fl_BlockLayout *>(pL);
+				fp_Run * pRun = pBL->findRunAtOffset(pcro->getBlockOffset());
+				if(pRun && pRun->getType() == FPRUN_BOOKMARK)
+				{
+					fp_BookmarkRun * pB = static_cast<fp_BookmarkRun*>(pRun);
+
+					// only do this when the end-object is deleted ...
+					// (in case of deletion, this is an arbitrary choice)
+					if(!pB->isStartOfBookmark())
+						sBookmark = pB->getName();
+				}
+				else if(pRun && pRun->getType() != FPRUN_BOOKMARK)
+				{
+					UT_ASSERT_HARMLESS( UT_SHOULD_NOT_HAPPEN );
+				}
+			}
+		}
+
+		// now we can proceed to delete the object representation ...
 		fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pL);
 		fl_SectionLayout* pCLSL = pCL->getSectionLayout();
 		if(pCLSL->getType() == FL_SECTION_SHADOW)
@@ -1209,6 +1470,11 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 		}
 		else
 			bResult = pCLSL->bl_doclistener_deleteObject(pCL, pcro);
+
+		// now notify TOCs if necessary
+		if(sBookmark.size())
+			m_pLayout->updateTOCsOnBookmarkChange(sBookmark.utf8_str());
+
 		goto finish_up;
 	}
 
@@ -1331,15 +1597,60 @@ bool fl_DocListener::change(PL_StruxFmtHandle sfh,
 	if(chgMask != AV_CHG_NONE)
 	{
 		UT_return_val_if_fail(getLayout() && getLayout()->getView(),bResult);
-		// first of all, increase view tick, so that the view's
-		// property caches are invalidated ...
-		getLayout()->getView()->incTick();
-		getLayout()->getView()->notifyListeners(chgMask);
+		if (m_bCacheChanges)
+		{
+			m_chgMaskCached |= chgMask;
+		}
+		else
+		{
+			UT_return_val_if_fail(getLayout() && getLayout()->getView(),bResult);
+			// first of all, increase view tick, so that the view's
+			// property caches are invalidated ...
+			getLayout()->getView()->incTick();
+			fl_ContainerLayout * pCL = (fl_ContainerLayout *)sfh;
+			bool doNotify = true;
+			if(pCL->isCollapsed())
+			{
+				doNotify = false;
+			}
+			if(doNotify)
+			{
+				UT_DEBUGMSG(("Doing view notification \n"));
+				getLayout()->getView()->notifyListeners(chgMask);
+			}
+			else
+			{
+				m_chgMaskCached |= chgMask;
+			}
+		}
 	}
 	
 	return bResult;
 }
 
+void fl_DocListener::deferNotifications(void)
+{
+	m_bCacheChanges = true;
+	m_chgMaskCached = AV_CHG_NONE;
+}
+
+void fl_DocListener::processDeferredNotifications(void)
+{
+	if (m_chgMaskCached != AV_CHG_NONE)
+	{
+		if (getLayout() && getLayout()->getView())
+		{
+			// first of all, increase view tick, so that the view's
+			// property caches are invalidated ...
+			getLayout()->getView()->incTick();
+			getLayout()->getView()->notifyListeners(m_chgMaskCached);
+		}
+		m_chgMaskCached = AV_CHG_NONE;
+	}
+	m_bCacheChanges = false;
+}
+
+
 /*!
  */
 bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
@@ -1351,7 +1662,9 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 														 PL_StruxFmtHandle sfhNew))
 {
 	UT_DEBUGMSG(("fl_DocListener::insertStrux at pos %d \n",pcr->getPosition()));
+	UT_return_val_if_fail( sdh && pcr, false );
 	UT_ASSERT(pcr->getType() == PX_ChangeRecord::PXT_InsertStrux);
+	
 	const PX_ChangeRecord_Strux * pcrx = static_cast<const PX_ChangeRecord_Strux *> (pcr);
 #if DEBUG
 #if 1
@@ -1371,6 +1684,14 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 	{
 		UT_DEBUGMSG(("Inserting EndFootnote strux at pos %d \n",pcr->getPosition()));
 	}
+	else if(pcrx->getStruxType() == PTX_SectionTOC)
+	{
+		UT_DEBUGMSG(("Inserting SectionTOC strux at pos %d \n",pcr->getPosition()));
+	}
+	else if(pcrx->getStruxType() == PTX_EndTOC)
+	{
+		UT_DEBUGMSG(("Inserting EndToc strux at pos %d \n",pcr->getPosition()));
+	}
 	else if(pcrx->getStruxType() == PTX_SectionEndnote)
 	{
 		UT_DEBUGMSG(("Inserting SectionEndnote strux at pos %d \n",pcr->getPosition()));
@@ -1386,6 +1707,8 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 	else if(pcrx->getStruxType() == PTX_EndTable)
 	{
 		UT_DEBUGMSG(("Inserting EndTable strux at pos %d \n",pcr->getPosition()));
+		m_pDoc->miniDump(sdh,8);
+
 	}
 	else if(pcrx->getStruxType() == PTX_SectionCell)
 	{
@@ -1395,6 +1718,14 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 	{
 		UT_DEBUGMSG(("Inserting EndCell strux at pos %d \n",pcr->getPosition()));
 	}
+	else if(pcrx->getStruxType() == PTX_SectionFrame)
+	{
+		UT_DEBUGMSG(("Inserting Frame strux at pos %d \n",pcr->getPosition()));
+	}
+	else if(pcrx->getStruxType() == PTX_EndFrame)
+	{
+		UT_DEBUGMSG(("Inserting EndFrame strux at pos %d \n",pcr->getPosition()));
+	}
 #endif
 #endif
 	fl_Layout * pL = (fl_Layout *)sfh;
@@ -1413,6 +1744,7 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 		case PTX_SectionHdrFtr:			//  ... or a HdrFtr section.
 		case PTX_SectionFootnote:        //  ... or a Footnote section.
 		case PTX_SectionEndnote:        //  ... or a Endnote section.
+		case PTX_SectionTOC:  //  ... or a Table Of Contents.
 			// We are inserting a section immediately after a section
 			// (with no intervening block).  This is probably a bug,
 			// because there should at least be an empty block between
@@ -1429,16 +1761,16 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 			fl_SectionLayout * pSL = static_cast<fl_SectionLayout *>(pL);
 			return pSL->bl_doclistener_insertBlock(NULL, pcrx,sdh,lid,pfnBindHandles);
 		}
-#if 0
+#if 1
 //
-// FIXME When I get brave I'll see if we can't make a table te first layout
+// FIXME When I get brave I'll see if we can't make a table the first layout
 // in a section.
 //
 		case PTX_SectionTable:
 		{
 			UT_DEBUGMSG(("Insert Table immediately after Section \n"));
 			fl_SectionLayout * pSL = static_cast<fl_SectionLayout *>(pL);
-			return pSL->bl_doclistener_insertTable(NULL, pcrx,sdh,lid,pfnBindHandles);
+			return pSL->bl_doclistener_insertTable(FL_SECTION_TABLE, pcrx,sdh,lid,pfnBindHandles);
 		}
 #endif
 		default:						// unknown strux.
@@ -1446,6 +1778,30 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 			return false;
 		}
 	}	
+	case PTX_SectionTOC:        //  ... or a TOC section.
+	{
+		switch (pcrx->getStruxType())	// see what we are inserting.
+		{
+		case PTX_EndTOC:
+		   {
+			   fl_TOCLayout * pCL = static_cast<fl_TOCLayout*>(pL);
+
+			   fl_DocSectionLayout* pDSL = 
+				   static_cast<fl_DocSectionLayout *>(pCL->myContainingLayout());
+			   UT_ASSERT(pCL->getContainerType() == FL_CONTAINER_TOC);
+			   UT_ASSERT(pDSL->getContainerType() == FL_CONTAINER_DOCSECTION);
+			   bool bResult = pCL->bl_doclistener_insertEndTOC(pCL, pcrx,sdh,lid,pfnBindHandles);
+			   m_pCurrentSL = static_cast<fl_SectionLayout *>(pDSL);
+			   return bResult;
+		   }
+		default:
+		   {
+			   m_pDoc->miniDump(pL->getStruxDocHandle(),6);
+			   UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			   return false;
+		   }
+		}
+	}
     case PTX_Block:						// the immediately prior strux is a block.
     {
 		switch (pcrx->getStruxType())	// see what we are inserting.
@@ -1501,6 +1857,14 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 			   bool bResult = pCLSL->bl_doclistener_insertSection(pCL, FL_SECTION_HDRFTR, pcrx,sdh,lid,pfnBindHandles);
 			   return bResult;
 		   }
+		case PTX_SectionTOC:
+		   {
+			   fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pL);
+			   fl_SectionLayout* pCLSL = pCL->getSectionLayout();
+
+			   bool bResult = pCLSL->bl_doclistener_insertSection(pCL, FL_SECTION_TOC, pcrx,sdh,lid,pfnBindHandles);
+			   return bResult;
+		   }
 		case PTX_SectionFootnote:
 		   {
 			   fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pL);
@@ -1511,6 +1875,7 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 		   }
 		case PTX_SectionEndnote:
 		   {
+		           UT_DEBUGMSG(("Doing valid endnote insertion into block \n"));
 			   fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pL);
 			   fl_SectionLayout* pCLSL = pCL->getSectionLayout();
 
@@ -1524,12 +1889,45 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 			   UT_DEBUGMSG(("Insert Table into Block \n"));
 			
 			   fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pL);
+			   UT_DEBUGMSG(("Containing Section is %d \n",pL->getType()));
 			   UT_DEBUGMSG(("Doing Insert Strux Table Into Prev Block \n"));
 			   xxx_UT_DEBUGMSG(("Doing Insert Table Correctly \n"));
 			   fl_SectionLayout* pCLSL = pCL->getSectionLayout();
 			   bool bResult = (pCLSL->bl_doclistener_insertTable(pCL,FL_SECTION_TABLE, pcrx,sdh,lid,pfnBindHandles) != 0);
 			   return bResult;
 		   }
+		case PTX_SectionFrame:				// we are inserting a section.
+		   {
+			   // The immediately prior strux is a block.  
+			   // OK this creates a table in the document.
+			   UT_DEBUGMSG(("Insert Frame after this Block \n"));
+			
+			   fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pL);
+			   UT_DEBUGMSG(("Doing Insert Strux Frame Into Prev Block \n"));
+			   fl_SectionLayout* pCLSL = pCL->getSectionLayout();
+			   bool bResult = (pCLSL->bl_doclistener_insertFrame(pCL,FL_SECTION_FRAME, pcrx,sdh,lid,pfnBindHandles) != 0);
+			   return bResult;
+		   }
+		case PTX_EndFrame:				// we are inserting an endFrame.
+		   {
+			   // The immediately prior strux is a block.  
+			   // OK this finishes a Frame on this page
+
+			   UT_DEBUGMSG(("Insert endFrame into Block \n"));
+			   fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pL);
+			   UT_DEBUGMSG(("Doing Insert Strux EndFrame adter Prev Block \n"));
+//
+// This gets us a fl_FRAMELAYOUTCell.
+//
+			   fl_FrameLayout* pCLSL = (fl_FrameLayout *) pCL->myContainingLayout();
+			   if(pCLSL->getContainerType() != FL_CONTAINER_FRAME)
+			   {
+				   m_pDoc->miniDump(pL->getStruxDocHandle(),6);
+			   }
+			   UT_ASSERT(pCLSL->getContainerType() == FL_CONTAINER_FRAME);
+			   bool bResult = pCLSL->bl_doclistener_insertEndFrame(pCL, pcrx,sdh,lid,pfnBindHandles);
+			   return bResult;
+		   }
 		case PTX_EndFootnote:
 		   {
 			   fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout*>(pL);
@@ -1599,7 +1997,7 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 			   // This isnerts a block after that. This happens on undo after
                // deleting a table.
 			   UT_DEBUGMSG(("Insert Block into Table/EndTable \n"));
-			
+			   m_pDoc->miniDump(pL->getStruxDocHandle(),3);			   
 			   fl_TableLayout * pTL = static_cast<fl_TableLayout *>(pL);
 			   bool bResult = pTL->bl_doclistener_insertBlock(NULL,pcrx,sdh,lid,pfnBindHandles);
 			   return bResult;
@@ -1638,6 +2036,17 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 			   bool bResult = pCLSL->bl_doclistener_insertEndCell(pCLSL, pcrx,sdh,lid,pfnBindHandles);
 			   return bResult;
 		   }
+		case PTX_SectionTOC:				// we are inserting a TOC.
+		{
+			   // The immediately prior strux is a Table. This can happen
+			// if were inserting a TOC after an EndTable Strux.
+            // We have write specal code to deal with it.
+
+			   fl_ContainerLayout * pCL = static_cast<fl_SectionLayout *>(pL);
+			   fl_SectionLayout * pCLSL = static_cast<fl_SectionLayout *>(pCL->myContainingLayout());
+			   bool bResult = pCLSL->bl_doclistener_insertSection(pCL, FL_SECTION_TOC, pcrx,sdh,lid,pfnBindHandles);
+			   return bResult;
+		}
 		default:
 		   {
 			   UT_DEBUGMSG(("Illegal strux type after table %d \n",pcrx->getStruxType()));
@@ -1765,8 +2174,135 @@ bool fl_DocListener::insertStrux(PL_StruxFmtHandle sfh,
 			  return bResult;
 		  }
 		default:
+		  {
+			  m_pDoc->miniDump(pL->getStruxDocHandle(),3);
+			  UT_DEBUGMSG(("Not Implemented \n"));
+			  UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			  break;
+		  }
+			
+		}
+
+	}
+    case PTX_SectionFrame:						// The immediately prior strux is a Frame
+    {
+		switch (pcrx->getStruxType())	// see what we are inserting.
+		{
+		case PTX_Block:				// we are inserting a block into a frame. This is valid
+		  {
+			   // The immediately prior strux is a Frame.  So, this
+			   // will become the first block of the Frame.
+
+			  UT_DEBUGMSG(("Inserting block into frame \n"));
+//
+// Actually the block could be being inserted right *after* the frame.
+// We need to detect this and deal with this. We can do this in fl_FrameLayout
+//
+			  fl_SectionLayout * pSL = static_cast<fl_SectionLayout *>(pL);
+			  UT_ASSERT(pSL->getContainerType() == FL_CONTAINER_FRAME);
+			  fl_FrameLayout * pFL = static_cast<fl_FrameLayout *>(pL);
+			  if(!pFL->isEndFrameIn())
+			  {
+				  bool bResult = pSL->bl_doclistener_insertBlock(NULL, pcrx,sdh,lid,pfnBindHandles);
+				  return bResult;
+			  }
+			  PT_DocPosition posEnd = pFL->getPosition(true) + pFL->getLength()-1;
+			  if(posEnd >= pcrx->getPosition())
+			  {
+				  bool bResult = pSL->bl_doclistener_insertBlock(NULL, pcrx,sdh,lid,pfnBindHandles);
+				  return bResult;
+			  }
+			  else
+			  {
+				  bool bResult = pFL->insertBlockAfter(NULL, pcrx,sdh,lid,pfnBindHandles);
+				  return bResult;
+
+			  }
+		   }
+		case PTX_SectionHdrFtr:
 		   {
-			   UT_DEBUGMSG(("Illegal strux type after cell %d \n",pcrx->getStruxType()));
+			   UT_DEBUGMSG(("Inserting HdrFtr after Frame \n"));
+			   fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pL);
+			   xxx_UT_DEBUGMSG(("Doing Insert Strux HdrFtr Into Prev Block \n"));
+			   fl_SectionLayout* pCLSL = pCL->getSectionLayout();
+			   bool bResult = pCLSL->bl_doclistener_insertSection(pCL, FL_SECTION_HDRFTR, pcrx,sdh,lid,pfnBindHandles);
+			   return bResult;
+			   
+		   }
+		case PTX_EndFrame:
+		   {
+			   UT_DEBUGMSG(("Inserting EndFrame immediately after Frame \n"));
+//
+// This gets us a fl_FrameLayout
+//
+			   fl_FrameLayout* pCLSL = static_cast<fl_FrameLayout *>( pL);
+			   if(pCLSL->getContainerType() != FL_CONTAINER_FRAME)
+			   {
+				   m_pDoc->miniDump(pL->getStruxDocHandle(),6);
+			   }
+			   UT_ASSERT(pCLSL->getContainerType() == FL_CONTAINER_FRAME);
+			   bool bResult = pCLSL->bl_doclistener_insertEndFrame(NULL, pcrx,sdh,lid,pfnBindHandles);
+			   return bResult;
+		   }
+		case PTX_SectionFrame:
+		   {
+			   UT_DEBUGMSG(("Inserting Frame immediately after Frame \n"));
+//
+// This gets us a fl_FrameLayout
+//
+			   fl_ContainerLayout* pCL = static_cast<fl_ContainerLayout *>( pL);
+			   if(pCL->getContainerType() != FL_CONTAINER_FRAME)
+			   {
+				   m_pDoc->miniDump(pL->getStruxDocHandle(),6);
+			   }
+			   UT_ASSERT(pCL->getContainerType() == FL_CONTAINER_FRAME);
+//
+// Loop back for the block
+//
+			   while(pCL && pCL->getContainerType() != FL_CONTAINER_BLOCK)
+			   {
+				   pCL = pCL->getPrev();
+			   }
+			   UT_return_val_if_fail(pCL != NULL,false); 
+			   UT_DEBUGMSG(("Doing Insert Strux Frame Into Prev Looped Block \n"));
+			   fl_SectionLayout* pCLSL = pCL->getSectionLayout();
+			   bool bResult = (pCLSL->bl_doclistener_insertFrame(pCL,FL_SECTION_FRAME, pcrx,sdh,lid,pfnBindHandles) != 0);
+			   return bResult;
+		   }
+		case PTX_SectionTable:	  // we are inserting a Table as the first layout into a frame. This is valid
+		  {
+			   // The immediately prior strux is a Frame.  So, this
+			   // will become the first table of the Frame.
+		           fl_SectionLayout * pSL = static_cast<fl_SectionLayout *>(pL);
+			   return pSL->bl_doclistener_insertTable(FL_SECTION_TABLE, pcrx,sdh,lid,pfnBindHandles);
+
+		  }
+		case PTX_EndCell:	  
+		  // we are inserting an endcell 
+		  // following an endframe. This is valid if the frame is
+		  // contained within the cell.
+		  {
+
+			   UT_DEBUGMSG(("Insert endCell after EndFrame \n"));
+			   fl_ContainerLayout * pCL = static_cast<fl_ContainerLayout *>(pL);
+			   UT_DEBUGMSG(("Doing Insert Strux EndCell after endFrame \n"));
+//
+// This gets us a fl_SectionCell.
+//
+			   UT_ASSERT(pCL->getContainerType() == FL_CONTAINER_FRAME);
+			   fl_CellLayout* pCLSL = static_cast<fl_CellLayout *>( pCL->myContainingLayout());
+			   if(pCLSL->getContainerType() != FL_CONTAINER_CELL)
+			   {
+				   m_pDoc->miniDump(pL->getStruxDocHandle(),6);
+			   }
+			   UT_return_val_if_fail(pCLSL->getContainerType() == FL_CONTAINER_CELL,false);
+			   bool bResult = pCLSL->bl_doclistener_insertEndCell(pCL, pcrx,sdh,lid,pfnBindHandles);
+			   return bResult;
+		  }
+
+		default:
+		   {
+			   UT_DEBUGMSG(("Illegal strux type after frame %d \n",pcrx->getStruxType()));
 			   m_pDoc->miniDump(pL->getStruxDocHandle(),6);
 			   UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 			   return false;
@@ -1805,12 +2341,23 @@ bool fl_DocListener::signal(UT_uint32 iSignal)
 	case PD_SIGNAL_REFORMAT_LAYOUT:
 		m_pLayout->formatAll();
 		break;
+	case PD_SIGNAL_REVISION_MODE_CHANGED:
+		pView->updateRevisionMode();
+		// fall through ...
+		
 	case PD_SIGNAL_DOCPROPS_CHANGED_REBUILD:
 		m_pLayout->updatePropsRebuild();
 		break;
 	case PD_SIGNAL_DOCPROPS_CHANGED_NO_REBUILD:
 		m_pLayout->updatePropsNoRebuild();
 		break;
+	case PD_SIGNAL_DOCNAME_CHANGED:
+		m_pLayout->notifyListeners(AV_CHG_FILENAME);
+		break;
+	case PD_SIGNAL_DOCDIRTY_CHANGED:
+		m_pLayout->notifyListeners(AV_CHG_DIRTY);
+		break;
+
 	default:
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 		break;
