@@ -660,7 +660,7 @@ void fp_VerticalContainer::getScreenOffsets(fp_ContainerObject* pContainer,
 		{
 		  bTable = true;
 		  pContainer = pCon;
-		  fp_Container * pCon = static_cast<fp_Container *>(this);
+		  pCon = static_cast<fp_Container *>(this);
 		  my_yoff = getY();
 		  my_xoff = getX();
 		}
@@ -1314,11 +1314,10 @@ void fp_VerticalContainer::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosit
 				// Note that it is difficult to prevent this from happening on the PT level, since
 				// just because text is marked as hidden or deleted does not mean it is not
 				// visible in a given view.
-				FL_DocLayout * pDL = NULL;
 				fp_Page * pPage = getPage();
-				if(getPage() && getPage()->getDocLayout() && getPage()->getDocLayout()->getFirstSection())
+				if(pPage && pPage->getDocLayout() && pPage->getDocLayout()->getFirstSection())
 				{
-					pBlock = getPage()->getDocLayout()->getFirstSection()->getFirstBlock();
+					pBlock = pPage->getDocLayout()->getFirstSection()->getFirstBlock();
 				}
 
 				if(!pBlock)
@@ -1995,6 +1994,8 @@ fp_ShadowContainer::fp_ShadowContainer(UT_sint32 iX,
 
 fp_ShadowContainer::~fp_ShadowContainer()
 {
+  UT_DEBUGMSG(("Delete Shadow Container %x from shadow Layout %x \n",this,getSectionLayout()));
+  getSectionLayout()->setFirstContainer(NULL);
 }
 
 
