@@ -59,7 +59,7 @@ fp_Line     * fp_Line::s_pMapOwner = 0;
 #endif
 
 #define STATIC_BUFFER_INITIAL 150
-
+#define INITIAL_LINE_OFFSET -99999999
 UT_sint32 * fp_Line::s_pOldXs = NULL;
 UT_uint32   fp_Line::s_iOldXsSize = 0;
 UT_uint32	fp_Line::s_iClassInstanceCounter = 0;
@@ -76,7 +76,7 @@ fp_Line::fp_Line(fl_SectionLayout * pSectionLayout) :
 	m_iAscent(0),
 	m_iDescent(0),
 	m_iX(0),
-	m_iY(-2000000), // So setY(0) triggers a clearscreen and redraw!
+	m_iY(INITIAL_LINE_OFFSET), // So setY(0) triggers a clearscreen and redraw!
 		            // I do not like this at all; we have no business
 		            // of clearing at fictional coordinances
 	//m_bRedoLayout(true),
@@ -87,7 +87,8 @@ fp_Line::fp_Line(fl_SectionLayout * pSectionLayout) :
 	m_bIsCleared(true),
 	m_bContainsFootnoteRef(false),
 	m_bIsWrapped(false),
-	m_bIsSameYAsPrevious(false)
+	m_bIsSameYAsPrevious(false),
+	m_iAdditionalMarginAfter(0)
 {
 	if(!s_iClassInstanceCounter)
 	{
@@ -381,9 +382,12 @@ void fp_Line::setMaxWidth(UT_sint32 iMaxWidth)
 {
 	if(iMaxWidth < 60) // This is hardwired to disallow -ve or too small values
 	{
+	  UT_DEBUGMSG(("Max width set to... %d \n",iMaxWidth));
+	  UT_ASSERT(0);
 		iMaxWidth = 60;
 	}
 	m_iMaxWidth = iMaxWidth;
+	xxx_UT_DEBUGMSG(("Line %x MaxWidth %d \n",this,iMaxWidth));
 	m_iClearToPos = iMaxWidth;
 	m_iClearLeftOffset = getDescent();
 }
@@ -458,6 +462,10 @@ bool fp_Line::removeRun(fp_Run* pRun, bool bTellTheRunAboutIt)
 //	if(pPrevRun)
 //		pPrevRun->clearScreen();
 
+        if(pRun->getType() == FPRUN_FORCEDPAGEBREAK)
+	{
+	    getBlock()->forceSectionBreak();
+	}
 	if (bTellTheRunAboutIt)
 	{
 		pRun->setLine(NULL);
@@ -1075,7 +1083,7 @@ void fp_Line::clearScreen(void)
 			UT_ASSERT(m_iClearToPos + m_iClearLeftOffset < getPage()->getWidth());
 //			pRun->Fill(getGraphics(),xoffLine - m_iClearLeftOffset, yoffLine, m_iClearToPos + m_iClearLeftOffset+iExtra, height);
 			pRun->Fill(getGraphics(),xoffLine - m_iClearLeftOffset, yoffLine, getMaxWidth() + m_iClearLeftOffset +iExtra, height);
-
+			xxx_UT_DEBUGMSG(("Clear pLine %x xoffline %d width %d \n",this,xoffLine,getMaxWidth() + m_iClearLeftOffset +iExtra));
 //
 // Sevior: I added this for robustness.
 //
@@ -1110,7 +1118,9 @@ void fp_Line::_doClearScreenFromRunToEnd(UT_sint32 runIndex)
 
 
 	// not sure what the reason for this is (Tomas, Oct 25, 2003)
-	fp_Run * pFRun = pRun;
+	fp_Run * pLeftVisualRun = pRun;
+	fp_Run * pRunToEraseFrom  = m_vecRuns.getNthItem(runIndex);
+	
 	bool bUseFirst = false;
 	
 	if(runIndex == 1)
@@ -1219,7 +1229,7 @@ void fp_Line::_doClearScreenFromRunToEnd(UT_sint32 runIndex)
 		}
 		if(bUseFirst)
 		{
-			getScreenOffsets(pFRun, xoff, yoff);
+			getScreenOffsets(pLeftVisualRun, xoff, yoff);
 		}
 		else
 		{
@@ -1233,7 +1243,7 @@ void fp_Line::_doClearScreenFromRunToEnd(UT_sint32 runIndex)
 		
 		fp_VerticalContainer * pVCon= (static_cast<fp_VerticalContainer *>(getContainer()));
 		pVCon->getScreenOffsets(this, xoffLine, yoffLine);
-
+		UT_sint32 diff = xoff - xoffLine;
 		UT_ASSERT(yoff == yoffLine);
 
 		fp_Line * pPrevLine = static_cast<fp_Line *>(getPrevContainerInSection());
@@ -1259,6 +1269,31 @@ void fp_Line::_doClearScreenFromRunToEnd(UT_sint32 runIndex)
 			xxx_UT_DEBUGMSG(("pl_Line _doClear no Page \n"));
 			return;
 		}
+
+		UT_sint32 iExtra = getGraphics()->tlu(2);
+
+		// this code adds half of the section/column margin to the left of the erasure are
+		// -- we can only do this if the run we are to erase from is the leftmost run
+		// (otherwise we erase area that belongs to runs that will not redraw themselves)
+
+		if(pLeftVisualRun == pRunToEraseFrom)
+		{
+			fl_DocSectionLayout * pSL =  getBlock()->getDocSectionLayout();
+			if(getContainer() &&
+			   (getContainer()->getContainerType() != FP_CONTAINER_CELL) &&
+			   (getContainer()->getContainerType() != FP_CONTAINER_FRAME))
+			{
+				if(pSL->getNumColumns() >1)
+				{
+					iExtra = pSL->getColumnGap()/2;
+				}
+				else
+				{
+					iExtra = pSL->getRightMargin()/2;
+				}
+			}
+		}
+		
 //		UT_ASSERT((m_iClearToPos + leftClear - (xoff-xoffLine)) <= getPage()->getWidth());
 		xxx_UT_DEBUGMSG(("Clear from runindex to end height %d \n",getHeight()));
 		xxx_UT_DEBUGMSG(("Width of clear %d \n",m_iClearToPos + leftClear - xoff));
@@ -1270,7 +1305,7 @@ void fp_Line::_doClearScreenFromRunToEnd(UT_sint32 runIndex)
 			// to the end of the line
 			pRun->Fill(getGraphics(), xoff - leftClear,
 					   yoff,
-					   m_iClearToPos + leftClear - xoff,
+					   getMaxWidth() + leftClear + iExtra -diff,
 					   getHeight());
 		}
 		else
@@ -1295,7 +1330,7 @@ void fp_Line::_doClearScreenFromRunToEnd(UT_sint32 runIndex)
 		if(bUseFirst)
 		{
 			//pRun = static_cast<fp_Run*>(m_vecRuns.getNthItem(_getRunLogIndx(0)));
-			pRun = pFRun;
+			pRun = pLeftVisualRun;
 			runIndex = 0;
 		}
 		pRun->markAsDirty();
@@ -2536,6 +2571,7 @@ void fp_Line::setX(UT_sint32 iX, bool bDontClearIfNeeded)
 	{
 		clearScreen();
 	}
+	xxx_UT_DEBUGMSG(("SetX Line %x X %d \n",this,iX));
 	m_iX = iX;
 }
 
@@ -2619,11 +2655,10 @@ UT_sint32 fp_Line::getMarginAfter(void) const
 			}
 		}
 		UT_sint32 iMargin = UT_MAX(iBottomMargin, iNextTopMargin);
-
-		return iMargin;
+		return iMargin + m_iAdditionalMarginAfter;
 	}
 
-	return 0;
+	return m_iAdditionalMarginAfter;
 }
 
 bool fp_Line::recalculateFields(UT_uint32 iUpdateCount)
@@ -2932,6 +2967,10 @@ bool	fp_Line::containsForcedColumnBreak(void) const
 		{
 			return true;
 		}
+		if(pRun->getPrevRun() && (pRun->getPrevRun()->getType() == FPRUN_FORCEDCOLUMNBREAK))
+		{
+			return true;
+		}
 	}
 
 	return false;
@@ -2946,10 +2985,24 @@ bool fp_Line::containsForcedPageBreak(void) const
 		{
 			return true;
 		}
+		if(pRun->getPrevRun() && (pRun->getPrevRun()->getType() == FPRUN_FORCEDPAGEBREAK))
+		{
+			return true;
+		}
 	}
 	return false;
 }
 
+/*!
+    Call to this function must be followed by calling fp_Line::layout() since coalescing
+    runs might require shaping of the resulting run, which in turn might result in lost of
+    justification information (this is due to the fact that the data used by the external
+    shaping engine can be opaque to us, so we are not able to combine it for the two
+    original runs -- we have to ask the external engine to recalculate it for the whole
+    combined run).
+
+    At the moment we simply call coalesceRuns() just before doing layout() in fb_LineBreaker().
+*/
 void fp_Line::coalesceRuns(void)
 {
 	//UT_DEBUGMSG(("coalesceRuns (line 0x%x)\n", this));
