@@ -3,7 +3,7 @@
 /* AbiWord
  * Copyright (C) 1998-2000 AbiSource, Inc.
  * Copyright (C) 2001-2005 Hubert Figuiere
- * Copyright (C) 2002-2004 Francis James Franklin
+ * Copyright (C) 2002-2005 Francis James Franklin
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -199,6 +199,10 @@ bool AP_CocoaApp::initialize(void)
 			path  = suffix;
 			path += "/Plug-ins";
 			s_createDirectoryIfNecessary(path.utf8_str(), true);
+
+			path  = suffix;
+			path += "/math";
+			s_createDirectoryIfNecessary(path.utf8_str(), true);
 		}
 
     const char * szUserPrivateDirectory = getUserPrivateDirectory();
@@ -244,6 +248,10 @@ bool AP_CocoaApp::initialize(void)
 														path2  = path;
 														path2 += "/Plug-ins";
 														s_createDirectoryIfNecessary(path2.utf8_str());
+
+														path2  = path;
+														path2 += "/math";
+														s_createDirectoryIfNecessary(path2.utf8_str());
 													}
 											}
 									}
@@ -416,6 +424,20 @@ bool AP_CocoaApp::initialize(void)
     return true;
 }
 
+void AP_CocoaApp::rebuildMenus(void)
+{
+	// getMenuFactory()->buildMenuLabelSet(m_szMenuLabelSetName);
+
+	DELETEP(m_pCocoaMenu);
+	m_pCocoaMenu = new EV_CocoaMenuBar(this, m_szMenuLayoutName, m_szMenuLabelSetName);
+	UT_ASSERT(m_pCocoaMenu);
+
+	m_pCocoaMenu->buildAppMenu();
+
+	XAP_CocoaAppController * pController = (XAP_CocoaAppController *) [NSApp delegate];
+	[pController reappendPluginMenuItems];
+}
+
 /*!
   Create a new frame based on the current one.  
   \return A pointer to the new frame.  
@@ -1182,7 +1204,7 @@ void AP_CocoaApp::catchSignals(int sig_num)
     UT_DEBUGMSG(("Oh no - we just segfaulted!\n"));
 	
     UT_uint32 i = 0;
-	IEFileType abiType = IE_Imp::fileTypeForSuffix("abw");
+	IEFileType abiType = IE_Imp::fileTypeForSuffix(".abw");
     for(;i<m_vecFrames.getItemCount();i++)
     {
 		AP_CocoaFrame * curFrame = (AP_CocoaFrame*) m_vecFrames[i];
