@@ -25,6 +25,8 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "ap_Features.h"
+
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
 #include "ut_exception.h"
@@ -109,6 +111,7 @@ void AP_Dialog_Options::_storeWindowData(void)
 	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 	// save the values to the Prefs classes
 	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_AutoSpellCheck, _gatherSpellCheckAsType() );
+	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_AutoGrammarCheck, _gatherGrammarCheck() );
 	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_SpellCheckCaps, _gatherSpellUppercase() );
 	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_SpellCheckNumbers, _gatherSpellNumbers() );
 	Save_Pref_Bool(pPrefsScheme, AP_PREF_KEY_ShowSplash,_gatherShowSplash());
@@ -270,11 +273,17 @@ void AP_Dialog_Options::_storeDataForControl (tControl id)
 
 	switch (id)
 	{
+
 		case id_CHECK_SPELL_CHECK_AS_TYPE:
 			Save_Pref_Bool (pPrefsScheme, AP_PREF_KEY_AutoSpellCheck,
 					_gatherSpellCheckAsType());
 			break;
 
+		case id_CHECK_GRAMMAR_CHECK:
+			Save_Pref_Bool (pPrefsScheme, AP_PREF_KEY_AutoGrammarCheck,
+					_gatherGrammarCheck());
+			break;
+
 		case id_CHECK_SPELL_UPPERCASE:
 			Save_Pref_Bool (pPrefsScheme, AP_PREF_KEY_SpellCheckCaps,
 					_gatherSpellUppercase());
@@ -479,6 +488,9 @@ void AP_Dialog_Options::_populateWindowData(void)
 	if (pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_SpellCheckNumbers,&b))
 		_setSpellNumbers (b);
 
+	if (pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_AutoGrammarCheck,&b))
+		_setGrammarCheck (b);
+
 	// ------------ Smart Quotes
 	if (pPrefs->getPrefsValueBool((XML_Char*)XAP_PREF_KEY_SmartQuotesEnable,&b))
 		_setSmartQuotesEnable (b);
@@ -693,7 +705,7 @@ AP_PreferenceScheme::AP_PreferenceScheme(AP_PreferenceSchemeManager * pSchemeMan
 	UT_uint32 i = 0;
 	UT_uint32 count = static_cast<UT_uint32>(bo__count);
 
-	for (UT_uint32 i = 0; i < count; i++)
+	for (i = 0; i < count; i++)
 		{
 			BoolOption bo = static_cast<BoolOption>(i);
 
@@ -702,7 +714,7 @@ AP_PreferenceScheme::AP_PreferenceScheme(AP_PreferenceSchemeManager * pSchemeMan
 		}
 	lookupDefaultOptionValues();
 
-	for (UT_uint32 i = 0; i < count; i++)
+	for (i = 0; i < count; i++)
 		{
 			BoolOption bo = static_cast<BoolOption>(i);
 
@@ -715,6 +727,8 @@ AP_PreferenceScheme::AP_PreferenceScheme(AP_PreferenceSchemeManager * pSchemeMan
 		m_BOData[bo_AutoSave		].m_original = bValue;
 	if (m_pPrefsScheme->getValueBool( AP_PREF_KEY_AutoSpellCheck,						&bValue))
 		m_BOData[bo_CheckSpelling	].m_original = bValue;
+	if (m_pPrefsScheme->getValueBool( AP_PREF_KEY_AutoGrammarCheck,						&bValue))
+		m_BOData[bo_CheckGrammar	].m_original = bValue;
 	if (m_pPrefsScheme->getValueBool( AP_PREF_KEY_CursorBlink,							&bValue))
 		m_BOData[bo_CursorBlink		].m_original = bValue;
 	if (m_pPrefsScheme->getValueBool(XAP_PREF_KEY_DirMarkerAfterClosingParenthesis,		&bValue))
@@ -769,7 +783,7 @@ AP_PreferenceScheme::AP_PreferenceScheme(AP_PreferenceSchemeManager * pSchemeMan
 	// NOT (YET?) IMPLEMENTED: if (m_pPrefsScheme->getValueBool("",&bValue)) m_BOData[bo_ViewAll			].m_original = bValue;
 	// NOT (YET?) IMPLEMENTED: if (m_pPrefsScheme->getValueBool("",&bValue)) m_BOData[bo_ViewHidden			].m_original = bValue;
 
-	for (UT_uint32 i = 0; i < count; i++)
+	for (i = 0; i < count; i++)
 		{
 			BoolOption bo = static_cast<BoolOption>(i);
 
@@ -938,6 +952,11 @@ void AP_PreferenceScheme::saveChanges()
 	if (m_BOData[bo].m_current != m_BOData[bo].m_original)
 		m_pPrefsScheme->setValueBool( AP_PREF_KEY_AutoSpellCheck,					 m_BOData[bo].m_current);
 
+
+	bo = bo_CheckGrammar;
+	if (m_BOData[bo].m_current != m_BOData[bo].m_original)
+		m_pPrefsScheme->setValueBool( AP_PREF_KEY_AutoGrammarCheck,					 m_BOData[bo].m_current);
+
 	bo = bo_CursorBlink;
 	if (m_BOData[bo].m_current != m_BOData[bo].m_original)
 		m_pPrefsScheme->setValueBool( AP_PREF_KEY_CursorBlink,						 m_BOData[bo].m_current);
@@ -1168,6 +1187,7 @@ void AP_PreferenceScheme::lookupDefaultOptionValues()
 
 	pScheme->getValueBool(XAP_PREF_KEY_AutoSaveFile,						&(m_BOData[bo_AutoSave			].m_default));
 	pScheme->getValueBool( AP_PREF_KEY_AutoSpellCheck,						&(m_BOData[bo_CheckSpelling		].m_default));
+	pScheme->getValueBool( AP_PREF_KEY_AutoGrammarCheck,						&(m_BOData[bo_CheckGrammar		].m_default));
 	pScheme->getValueBool( AP_PREF_KEY_CursorBlink,							&(m_BOData[bo_CursorBlink		].m_default));
 	pScheme->getValueBool(XAP_PREF_KEY_DirMarkerAfterClosingParenthesis,	&(m_BOData[bo_DirectionMarkers	].m_default));
 	pScheme->getValueBool( AP_PREF_KEY_DefaultDirectionRtl,					&(m_BOData[bo_DirectionRTL		].m_default));
