@@ -71,6 +71,7 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_FILE_NEW,			0,0,0,0,	"fileNew",			NULL,					NULL);
 	_s(AP_MENU_ID_FILE_NEW_USING_TEMPLATE,			0,1,0,0,	"fileNewUsingTemplate",			NULL,					NULL);
 	_s(AP_MENU_ID_FILE_OPEN,		0,1,0,0,	"fileOpen",			NULL,					NULL);
+	_s(AP_MENU_ID_FILE_IMPORTSTYLES,		0,1,0,0,	"importStyles",	NULL,					NULL);
 	_s(AP_MENU_ID_FILE_SAVE,		0,0,0,0,	"fileSave",			ap_GetState_Changes,					NULL);
 	_s(AP_MENU_ID_FILE_SAVEAS,		0,1,0,0,	"fileSaveAs",		NULL,					NULL);
 	_s(AP_MENU_ID_FILE_SAVEIMAGE,		0,1,0,0,	"fileSaveImage",		NULL,					NULL);
@@ -83,10 +84,10 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_FILE_PRINT,		0,1,0,0,	"print",			NULL,					NULL);
 	_s(AP_MENU_ID_FILE_PRINT_PREVIEW, 0,1,0,0, "printPreview", NULL, NULL);
 	_s(AP_MENU_ID_FILE_RECENT,	1,0,0,0,	NULL, ap_GetState_Recent,NULL);
-	_s(AP_MENU_ID_FILE_RECENT_1,	0,0,0,0,	"openRecent_1",		NULL,					ap_GetLabel_Recent);
-	_s(AP_MENU_ID_FILE_RECENT_2,	0,0,0,0,	"openRecent_2",		NULL,					ap_GetLabel_Recent);
-	_s(AP_MENU_ID_FILE_RECENT_3,	0,0,0,0,	"openRecent_3",		NULL,					ap_GetLabel_Recent);
-	_s(AP_MENU_ID_FILE_RECENT_4,	0,0,0,0,	"openRecent_4",		NULL,					ap_GetLabel_Recent);
+ 	_s(AP_MENU_ID_FILE_RECENT_1,	0,0,0,0,	"openRecent_1",		NULL,					ap_GetLabel_Recent);
+ 	_s(AP_MENU_ID_FILE_RECENT_2,	0,0,0,0,	"openRecent_2",		NULL,					ap_GetLabel_Recent);
+ 	_s(AP_MENU_ID_FILE_RECENT_3,	0,0,0,0,	"openRecent_3",		NULL,					ap_GetLabel_Recent);
+ 	_s(AP_MENU_ID_FILE_RECENT_4,	0,0,0,0,	"openRecent_4",		NULL,					ap_GetLabel_Recent);
 	_s(AP_MENU_ID_FILE_RECENT_5,	0,0,0,0,	"openRecent_5",		NULL,					ap_GetLabel_Recent);
 	_s(AP_MENU_ID_FILE_RECENT_6,	0,0,0,0,	"openRecent_6",		NULL,					ap_GetLabel_Recent);
 	_s(AP_MENU_ID_FILE_RECENT_7,	0,0,0,0,	"openRecent_7",		NULL,					ap_GetLabel_Recent);
@@ -94,12 +95,14 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_FILE_RECENT_9,	0,0,0,0,	"openRecent_9",		NULL,					ap_GetLabel_Recent);
 	_s(AP_MENU_ID_FILE_REVERT, 0,0,0,0, "fileRevert", ap_GetState_Changes, NULL);
 	_s(AP_MENU_ID_FILE_EXIT,		0,0,0,0,	"querySaveAndExit",	NULL,					NULL);
+	_s(AP_MENU_ID_FILE_SAVEEMBED,		0,0,0,0,	"fileSaveEmbed",	NULL,					NULL);
 	_s(AP_MENU_ID_OPEN_TEMPLATE, 0,1,0,0, "openTemplate", NULL, NULL);
 
 	_s(AP_MENU_ID_EDIT,				1,0,0,0,	NULL,				NULL,					NULL);
 	_s(AP_MENU_ID_EDIT_UNDO,		0,0,0,0,	"undo",				ap_GetState_Changes,	NULL);
 	_s(AP_MENU_ID_EDIT_REDO,		0,0,0,0,	"redo",				ap_GetState_Changes,	NULL);
 	_s(AP_MENU_ID_EDIT_CUT,			0,0,0,0,	"cut",				ap_GetState_Selection,	NULL);
+	_s(AP_MENU_ID_EDIT_LATEXEQUATION,			0,0,0,0,	"editLatexAtPos",				ap_GetState_Selection,	NULL);
 	_s(AP_MENU_ID_EDIT_COPY,		0,0,0,0,	"copy",				ap_GetState_Selection,	NULL);
 	_s(AP_MENU_ID_EDIT_PASTE,		0,0,0,0,	"paste",			ap_GetState_Clipboard,	NULL);
 	_s(AP_MENU_ID_EDIT_PASTE_SPECIAL,0,0,0,0, "pasteSpecial",     ap_GetState_Clipboard,  NULL);
@@ -113,9 +116,16 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_EDIT_REMOVEHEADER,		0,0,0,0,"removeHeader",ap_GetState_Changes	,					NULL);
 	_s(AP_MENU_ID_EDIT_REMOVEFOOTER,		0,0,0,0,	"removeFooter",ap_GetState_Changes,					NULL);
 	_s(AP_MENU_ID_EDIT_DELETEFRAME,		0,0,0,0,	"deleteFrame", NULL,NULL);
+	_s(AP_MENU_ID_EDIT_DELETEIMAGE,		0,0,0,0,	"deleteFrame", NULL,NULL);
+	_s(AP_MENU_ID_FMT_POSIMAGE,		0,1,0,0,	"dlgFmtPosImage", NULL,NULL);
+	_s(AP_MENU_ID_EDIT_CUTIMAGE,		0,0,0,0,	"cut", NULL,NULL);
+	_s(AP_MENU_ID_EDIT_COPYIMAGE,		0,0,0,0,	"copy", NULL,NULL);
 	_s(AP_MENU_ID_EDIT_CUT_FRAME,		0,0,0,0,	"cutFrame", NULL,NULL);
 	_s(AP_MENU_ID_EDIT_COPY_FRAME,		0,0,0,0,	"copyFrame",NULL,NULL);
 	_s(AP_MENU_ID_EDIT_SELECT_FRAME,	0,0,0,0,	"selectFrame",NULL,NULL);
+	_s(AP_MENU_ID_EDIT_CUTEMBED,	0,0,0,0,	"cut",NULL,NULL);
+	_s(AP_MENU_ID_EDIT_COPYEMBED,	0,0,0,0,	"copy",NULL,NULL);
+	_s(AP_MENU_ID_EDIT_DELETEEMBED,	0,0,0,0,	"delLeft",NULL,NULL);
 
 	_s(AP_MENU_ID_VIEW,				1,0,0,0,	NULL,				NULL,					NULL);
 	_s(AP_MENU_ID_VIEW_NORMAL, 0,0,0,1, "viewNormalLayout", ap_GetState_View, NULL);
@@ -148,6 +158,7 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_INSERT_HYPERLINK, 0,1,0,0,	"insertHyperlink",	ap_GetState_HyperlinkOK,					NULL);
 	_s(AP_MENU_ID_INSERT_GOTO_HYPERLINK, 0,1,0,0,	"hyperlinkJumpPos",	ap_GetState_HyperlinkOK,					NULL);
 	_s(AP_MENU_ID_INSERT_EDIT_HYPERLINK, 0,1,0,0,	"insertHyperlink",	ap_GetState_HyperlinkOK,					NULL);
+	_s(AP_MENU_ID_EDIT_COPY_HYPERLINK_LOCATION, 0,0,0,0, "hyperlinkCopyLocation", NULL, NULL);
 	_s(AP_MENU_ID_INSERT_DELETE_HYPERLINK, 0,0,0,0,	"deleteHyperlink",	NULL,			NULL);
 	_s(AP_MENU_ID_INSERT_PAGENO,	0,1,0,0,	"insPageNo",		NULL,					NULL);
 	_s(AP_MENU_ID_INSERT_DATETIME,	0,1,0,0,	"insDateTime",		NULL,					NULL);
@@ -191,9 +202,9 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_FMT_TABLEOFCONTENTS, 0,1,0,0, "formatTOC", ap_GetState_InTOC, NULL);
 	_s(AP_MENU_ID_FMT_FOOTNOTES,     0,1,0,0, "formatFootnotes", NULL, NULL);
 	_s(AP_MENU_ID_FMT_IMAGE, 0,1,0,0, "dlgFmtImage", ap_GetState_InImage, NULL);
+	_s(AP_MENU_ID_FMT_SETPOSIMAGE, 0,0,0,0, "setPosImage", ap_GetState_SetPosImage, NULL);
 	_s(AP_MENU_ID_FMT_STYLE_DEFINE,		0,1,0,0,	"dlgStyle",			NULL,					NULL);
 	_s(AP_MENU_ID_FMT_STYLE,		1,0,0,0,	NULL,			NULL,					NULL);
-	_s(AP_MENU_ID_FMT_IMPORTSTYLES,		0,1,0,0,	"importStyles",	NULL,					NULL);
 	_s(AP_MENU_ID_FMT_STYLIST,		0,1,0,0,	"dlgStylist",	NULL,					NULL);
 	_s(AP_MENU_ID_FMT_TABS,			0,1,0,0,	"dlgTabs",			NULL,					NULL);
 	_s(AP_MENU_ID_FMT_BOLD,			0,0,1,0,	"toggleBold",		ap_GetState_CharFmt,	NULL);
@@ -212,6 +223,7 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_FMT_DIRECTION_DD_RTL,0,0,1,0, "toggleDomDirection", ap_GetState_BlockFmt, NULL);
 	_s(AP_MENU_ID_FMT_DIRECTION_DO_LTR,0,0,1,0, "toggleDirOverrideLTR", ap_GetState_CharFmt, NULL);
 	_s(AP_MENU_ID_FMT_DIRECTION_DO_RTL,0,0,1,0, "toggleDirOverrideRTL", ap_GetState_CharFmt, NULL);
+	_s(AP_MENU_ID_FMT_EMBED,0,1,0,0, "editEmbed", NULL, NULL);
 
 	_s(AP_MENU_ID_ALIGN,			1,0,0,0,	NULL,				ap_GetState_StylesLocked,	NULL);
 	_s(AP_MENU_ID_ALIGN_LEFT,		0,0,0,1,	"alignLeft",		ap_GetState_BlockFmt,	NULL);
@@ -231,7 +243,8 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_TOOLS_SCRIPTS,	0,1,0,0,	"scriptPlay", ap_GetState_ScriptsActive, NULL);
 	_s(AP_MENU_ID_TOOLS_MAILMERGE,	0,1,0,0,	"mailMerge", ap_GetState_MailMerge, NULL);
 	_s(AP_MENU_ID_TOOLS_REVISIONS,  1,0,0,0,  NULL,               NULL,                   NULL);
-	_s(AP_MENU_ID_TOOLS_REVISIONS_MARK, 0,0,1,0, "toggleMarkRevisions", ap_GetState_MarkRevisions,NULL);
+	_s(AP_MENU_ID_TOOLS_REVISIONS_MARK, 0,0,1,0, "toggleMarkRevisions", ap_GetState_MarkRevisionsCheck,NULL);
+	_s(AP_MENU_ID_TOOLS_REVISIONS_NEW_REVISION, 0,0,0,0, "startNewRevision", ap_GetState_MarkRevisions,NULL);
 	_s(AP_MENU_ID_TOOLS_REVISIONS_SHOW, 0,0,1,0, "toggleShowRevisions", ap_GetState_ShowRevisions,NULL);
 	_s(AP_MENU_ID_TOOLS_REVISIONS_SHOW_AFTERPREV, 0,0,1,0, "toggleShowRevisionsAfterPrevious", ap_GetState_ShowRevisionsAfterPrev,NULL);
 	_s(AP_MENU_ID_TOOLS_REVISIONS_SHOW_AFTER, 0,0,1,0, "toggleShowRevisionsAfter", ap_GetState_ShowRevisionsAfter,NULL);
@@ -239,6 +252,7 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	
 	_s(AP_MENU_ID_TOOLS_REVISIONS_ACCEPT_REVISION, 0,0,0,0, "revisionAccept", ap_GetState_RevisionPresent,NULL);
 	_s(AP_MENU_ID_TOOLS_REVISIONS_REJECT_REVISION, 0,0,0,0, "revisionReject", ap_GetState_RevisionPresent,NULL);
+	_s(AP_MENU_ID_TOOLS_REVISIONS_PURGE, 0,0,0,0, "purgeAllRevisions", ap_GetState_HasRevisions,NULL);
 	_s(AP_MENU_ID_TOOLS_REVISIONS_COMPARE_DOCUMENTS, 0,1,0,0, "revisionCompareDocuments",NULL,NULL);
 	_s(AP_MENU_ID_TOOLS_REVISIONS_MERGE_DOCUMENTS, 0,1,0,0, "revisionMergeDocuments",NULL,NULL);
 	_s(AP_MENU_ID_TOOLS_REVISIONS_AUTO, 0,0,1,0, "toggleAutoRevision",ap_GetState_AutoRevision,NULL);
@@ -258,19 +272,23 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_TABLE_INSERT_CELLS,0,0,0,0, NULL, ap_GetState_InTable, NULL);
 	_s(AP_MENU_ID_TABLE_INSERT_SUMROWS,0,0,0,0, "insertSumRows", ap_GetState_InTable, NULL);
 	_s(AP_MENU_ID_TABLE_INSERT_SUMCOLS,0,0,0,0, "insertSumCols", ap_GetState_InTable, NULL);
-	_s(AP_MENU_ID_TABLE_DELETE,1,0,0,0, NULL, ap_GetState_InTable, NULL);
-	_s(AP_MENU_ID_TABLE_DELETE_TABLE,0,0,0,0, "deleteTable", ap_GetState_InTable, NULL);
-	_s(AP_MENU_ID_TABLE_DELETETABLE,0,0,0,0, "deleteTable", ap_GetState_InTable, NULL);
+	_s(AP_MENU_ID_TABLE_DELETE,1,0,0,0, NULL, ap_GetState_PointOrAnchorInTable, NULL);
+	_s(AP_MENU_ID_TABLE_DELETE_TABLE,0,0,0,0, "deleteTable", ap_GetState_PointOrAnchorInTable, NULL);
+	_s(AP_MENU_ID_TABLE_DELETETABLE,0,0,0,0, "deleteTable", ap_GetState_PointOrAnchorInTable, NULL);
 	_s(AP_MENU_ID_TABLE_DELETE_COLUMNS,0,0,0,0, "deleteColumns", ap_GetState_InTable, NULL);
 	_s(AP_MENU_ID_TABLE_DELETECOLUMN,0,0,0,0, "deleteColumns", ap_GetState_InTable, NULL);
 	_s(AP_MENU_ID_TABLE_DELETE_ROWS,0,0,0,0, "deleteRows", ap_GetState_InTable, NULL);
 	_s(AP_MENU_ID_TABLE_DELETEROW,0,0,0,0, "deleteRows", ap_GetState_InTable, NULL);
 	_s(AP_MENU_ID_TABLE_DELETE_CELLS,0,0,0,0, "deleteCell", ap_GetState_InTable, NULL);
-	_s(AP_MENU_ID_TABLE_TEXTTOTABLE,1,0,0,0, "textToTable", ap_GetState_TextToTableOK, NULL);
+	_s(AP_MENU_ID_TABLE_TEXTTOTABLE_ALL,1,0,0,0, "textToTable", ap_GetState_TextToTableOK, NULL);
+	_s(AP_MENU_ID_TABLE_TEXTTOTABLE_NOSPACES,1,0,0,0, "textToTableNoSpaces", ap_GetState_TextToTableOK, NULL);
 	_s(AP_MENU_ID_TABLE_SORTROWSASCEND,1,0,0,0, "sortRowsAscend", ap_GetState_InTable, NULL);
 	_s(AP_MENU_ID_TABLE_SORTROWSDESCEND,1,0,0,0, "sortRowsDescend", ap_GetState_InTable, NULL);
 	_s(AP_MENU_ID_TABLE_SORTCOLSASCEND,1,0,0,0, "sortColsAscend", ap_GetState_InTable, NULL);
 	_s(AP_MENU_ID_TABLE_SORTCOLSDESCEND,1,0,0,0, "sortColsDescend", ap_GetState_InTable, NULL);
+	_s(AP_MENU_ID_TABLE_TABLETOTEXTCOMMAS,1,0,0,0, "tableToTextCommas", ap_GetState_PointOrAnchorInTable, NULL);
+	_s(AP_MENU_ID_TABLE_TABLETOTEXTTABS,1,0,0,0, "tableToTextTabs", ap_GetState_PointOrAnchorInTable, NULL);
+	_s(AP_MENU_ID_TABLE_TABLETOTEXTCOMMASTABS,1,0,0,0, "tableToTextCommasTabs", ap_GetState_PointOrAnchorInTable, NULL);
 	_s(AP_MENU_ID_TABLE_SELECT,1,0,0,0, NULL, ap_GetState_InTable, NULL);
 	_s(AP_MENU_ID_TABLE_SELECT_TABLE,0,0,0,0, "selectTable", ap_GetState_InTable, NULL);
 	_s(AP_MENU_ID_TABLE_SELECT_COLUMN,0,0,0,0, "selectColumn", ap_GetState_InTable, NULL);
@@ -279,10 +297,14 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_TABLE_MERGE_CELLS,0,1,0,0, "mergeCells", ap_GetState_InTable, NULL);
 	_s(AP_MENU_ID_TABLE_SPLIT_CELLS,0,1,0,0, "splitCells", ap_GetState_InTableMerged, NULL);
 	_s(AP_MENU_ID_TABLE_SPLIT_TABLE,0,0,0,0, NULL, ap_GetState_AlwaysDisabled, NULL);
-	_s(AP_MENU_ID_TABLE_FORMAT,0,1,0,0, "formatTable", ap_GetState_InTable, NULL);
-	_s(AP_MENU_ID_TABLE_AUTOFIT,0,0,0,0, "autoFitTable", ap_GetState_InTable, NULL);
-	_s(AP_MENU_ID_TABLE_HEADING_ROWS_REPEAT,0,0,0,0, NULL, ap_GetState_AlwaysDisabled, NULL);
-	_s(AP_MENU_ID_TABLE_SORT,0,0,0,0, NULL, ap_GetState_InTable, NULL);
+	_s(AP_MENU_ID_TABLE_FORMAT,0,1,0,0, "formatTable", ap_GetState_PointOrAnchorInTable, NULL);
+	_s(AP_MENU_ID_TABLE_AUTOFIT,0,0,0,0, "autoFitTable", ap_GetState_PointInTable, NULL);
+	_s(AP_MENU_ID_TABLE_TABLETOTEXT,0,0,0,0, NULL, ap_GetState_PointInTable, NULL);
+	_s(AP_MENU_ID_TABLE_HEADING_ROWS_REPEAT,0,0,0,0, NULL, ap_GetState_PointInTable, NULL);
+	_s(AP_MENU_ID_TABLE_HEADING_ROWS_REPEAT_THIS,0,0,0,0, "repeatThisRow", ap_GetState_PointInTable, NULL);
+	_s(AP_MENU_ID_TABLE_HEADING_ROWS_REPEAT_REMOVE,0,0,0,0, "removeThisRowRepeat", ap_GetState_InTableIsRepeat, NULL);
+	_s(AP_MENU_ID_TABLE_SORT,0,0,0,0, NULL, ap_GetState_PointOrAnchorInTable, NULL);
+	_s(AP_MENU_ID_TABLE_TEXTTOTABLE,0,0,0,0, NULL, ap_GetState_TextToTableOK, NULL);
 
 
 	_s(AP_MENU_ID_CONTEXT_REVISIONS_ACCEPT_REVISION, 0,0,0,0, "revisionAccept", ap_GetState_RevisionPresent,NULL);
@@ -292,6 +314,7 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_TOOLS_REVISIONS_SET_VIEW_LEVEL, 0,1,0,0, "revisionSetViewLevel", ap_GetState_RevisionsSelectLevel, NULL);
 	_s(AP_MENU_ID_TOOLS_HISTORY,  1,0,0,0,  NULL,               NULL,                   NULL);
 	_s(AP_MENU_ID_TOOLS_HISTORY_SHOW, 0,1,0,0, "history", ap_GetState_History, NULL);
+	_s(AP_MENU_ID_TOOLS_HISTORY_PURGE, 0,0,0,0, "purgeAllRevisions", ap_GetState_HasRevisions, NULL);
 
 	_s(AP_MENU_ID_WINDOW,			1,0,0,0,	NULL,				NULL,					NULL);
 	_s(AP_MENU_ID_WINDOW_NEW,		0,0,0,0,	"newWindow",		NULL,					NULL);
@@ -315,8 +338,6 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_HELP_CHECKVER,		0,0,0,0,	"helpCheckVer",			NULL,					ap_GetLabel_Checkver);
 	_s(AP_MENU_ID_HELP_SEARCH,		0,0,0,0,	"helpSearch",			NULL,					ap_GetLabel_Search);
 	_s(AP_MENU_ID_HELP_ABOUT,		0,1,0,0,	"dlgAbout",			NULL,					ap_GetLabel_About);
-	_s(AP_MENU_ID_HELP_ABOUTOS,		0,0,0,0,	"helpAboutOS",			NULL,					ap_GetLabel_AboutOS);
-	_s(AP_MENU_ID_HELP_ABOUT_GNU, 0,0,0,0, "helpAboutGnu", NULL, NULL);
 	_s(AP_MENU_ID_HELP_ABOUT_GNOMEOFFICE, 0,0,0,0, "helpAboutGnomeOffice", NULL, NULL);
 	_s(AP_MENU_ID_HELP_CREDITS, 0,0,0,0, "helpCredits", NULL, NULL);
 	_s(AP_MENU_ID_HELP_REPORT_BUG, 0,0,0,0, "helpReportBug", NULL, NULL);
@@ -333,58 +354,6 @@ EV_Menu_ActionSet * AP_CreateMenuActionSet(void)
 	_s(AP_MENU_ID_SPELL_IGNOREALL,	0,0,0,0,	"spellIgnoreAll",	NULL,					NULL);
 	_s(AP_MENU_ID_SPELL_ADD,		0,0,0,0,	"spellAdd",			NULL,					NULL);
 
-	_s(AP_MENU_ID_INSERT_AUTOTEXT,  1,0,0,0, NULL, NULL, NULL);
-	_s(AP_MENU_ID_AUTOTEXT_ATTN, 1,0,0,0, NULL, NULL, NULL);
-	_s(AP_MENU_ID_AUTOTEXT_CLOSING, 1,0,0,0, NULL, NULL, NULL);
-	_s(AP_MENU_ID_AUTOTEXT_MAIL, 1,0,0,0, NULL, NULL, NULL);
-	_s(AP_MENU_ID_AUTOTEXT_REFERENCE, 1,0,0,0, NULL, NULL, NULL);
-	_s(AP_MENU_ID_AUTOTEXT_SALUTATION, 1,0,0,0, NULL, NULL, NULL);
-	_s(AP_MENU_ID_AUTOTEXT_SUBJECT, 1,0,0,0, NULL, NULL, NULL);
-	_s(AP_MENU_ID_AUTOTEXT_EMAIL, 1,0,0,0, NULL, NULL, NULL);
-
-	_s(AP_MENU_ID_AUTOTEXT_ATTN_1, 0,0,0,0, "insAutotext_attn_1", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_ATTN_2, 0,0,0,0, "insAutotext_attn_2", NULL, ap_GetLabel_Autotext);
-
-	_s(AP_MENU_ID_AUTOTEXT_CLOSING_1, 0,0,0,0, "insAutotext_closing_1", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_CLOSING_2, 0,0,0,0, "insAutotext_closing_2", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_CLOSING_3, 0,0,0,0, "insAutotext_closing_3", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_CLOSING_4, 0,0,0,0, "insAutotext_closing_4", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_CLOSING_5, 0,0,0,0, "insAutotext_closing_5", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_CLOSING_6, 0,0,0,0, "insAutotext_closing_6", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_CLOSING_7, 0,0,0,0, "insAutotext_closing_7", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_CLOSING_8, 0,0,0,0, "insAutotext_closing_8", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_CLOSING_9, 0,0,0,0, "insAutotext_closing_9", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_CLOSING_10, 0,0,0,0, "insAutotext_closing_10", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_CLOSING_11, 0,0,0,0, "insAutotext_closing_11", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_CLOSING_12, 0,0,0,0, "insAutotext_closing_12", NULL, ap_GetLabel_Autotext);
-
-	_s(AP_MENU_ID_AUTOTEXT_MAIL_1, 0,0,0,0, "insAutotext_mail_1", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_MAIL_2, 0,0,0,0, "insAutotext_mail_2", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_MAIL_3, 0,0,0,0, "insAutotext_mail_3", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_MAIL_4, 0,0,0,0, "insAutotext_mail_4", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_MAIL_5, 0,0,0,0, "insAutotext_mail_5", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_MAIL_6, 0,0,0,0, "insAutotext_mail_6", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_MAIL_7, 0,0,0,0, "insAutotext_mail_7", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_MAIL_8, 0,0,0,0, "insAutotext_mail_8", NULL, ap_GetLabel_Autotext);
-
-	_s(AP_MENU_ID_AUTOTEXT_REFERENCE_1, 0,0,0,0, "insAutotext_reference_1", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_REFERENCE_2, 0,0,0,0, "insAutotext_reference_2", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_REFERENCE_3, 0,0,0,0, "insAutotext_reference_3", NULL, ap_GetLabel_Autotext);
-
-	_s(AP_MENU_ID_AUTOTEXT_SALUTATION_1, 0,0,0,0, "insAutotext_salutation_1", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_SALUTATION_2, 0,0,0,0, "insAutotext_salutation_2", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_SALUTATION_3, 0,0,0,0, "insAutotext_salutation_3", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_SALUTATION_4, 0,0,0,0, "insAutotext_salutation_4", NULL, ap_GetLabel_Autotext);
-
-	_s(AP_MENU_ID_AUTOTEXT_SUBJECT_1, 0,0,0,0, "insAutotext_subject_1", NULL, ap_GetLabel_Autotext);
-
-	_s(AP_MENU_ID_AUTOTEXT_EMAIL_1, 0,0,0,0, "insAutotext_email_1", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_EMAIL_2, 0,0,0,0, "insAutotext_email_2", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_EMAIL_3, 0,0,0,0, "insAutotext_email_3", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_EMAIL_4, 0,0,0,0, "insAutotext_email_4", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_EMAIL_5, 0,0,0,0, "insAutotext_email_5", NULL, ap_GetLabel_Autotext);
-	_s(AP_MENU_ID_AUTOTEXT_EMAIL_6, 0,0,0,0, "insAutotext_email_6", NULL, ap_GetLabel_Autotext);
-
 	// ... add others here ...
 
 	_s(AP_MENU_ID__BOGUS2__,		0,0,0,0,	NULL,				NULL,					NULL);
