@@ -35,7 +35,7 @@
 #include "ie_types.h"
 #include "ie_impGraphic.h"
 #include "fl_FrameLayout.h"
-
+#include "ut_rand.h"
 #include "fg_Graphic.h"
 #include "fg_GraphicRaster.h"
 #include "fg_GraphicVector.h"
@@ -193,6 +193,10 @@ bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
 	UT_String propBuffer;
 	double wInch = 0.0f;
 	double hInch = 0.0f;
+    double cropt = 0.0f;
+	double cropb = 0.0f;
+	double cropl = 0.0f;
+	double cropr = 0.0f;
 	bool resize = false;
 	if (!bUseInsertNotAppend())
 	{
@@ -227,13 +231,22 @@ bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
 			break;
 		}
 
+		if ( imgProps.bCrop )
+		{
+			cropt = imgProps.cropt / 1440.0f;
+			cropb = imgProps.cropb / 1440.0f;
+			cropl = imgProps.cropl / 1440.0f;
+			cropr = imgProps.cropr / 1440.0f;
+			resize = true;
+		}
+	  
 		if (resize) 
 		{
-			UT_LocaleTransactor(LC_NUMERIC, "C");
+			UT_LocaleTransactor t(LC_NUMERIC, "C");
 			UT_DEBUGMSG (("resizing...\n"));
-			UT_String_sprintf(propBuffer, "width:%fin; height:%fin",
-							  wInch, hInch);
-			xxx_UT_DEBUGMSG (("props are %s\n", propBuffer.c_str()));
+			UT_String_sprintf(propBuffer, "width:%fin; height:%fin; cropt:%fin; cropb:%fin; cropl:%fin; cropr:%fin",
+							  wInch, hInch, cropt, cropb, cropl, cropr);
+			UT_DEBUGMSG (("props are %s\n", propBuffer.c_str()));
 		}
 
 		const XML_Char* propsArray[5];
@@ -337,8 +350,14 @@ bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
 		case RTFProps_ImageProps::ipstScale:
 			UT_DEBUGMSG (("Scale: x=%d, y=%d, w=%d, h=%d\n", imgProps.scaleX, imgProps.scaleY, imgProps.width, imgProps.height));
 			resize = true;
-			wInch = ((static_cast<double>(imgProps.scaleX) / 100.0f) * imgProps.width);
-			hInch = ((static_cast<double>(imgProps.scaleY) / 100.0f) * imgProps.height);
+			if ((imgProps.wGoal != 0) && (imgProps.hGoal != 0)) {
+				// want image scaled against the w&h specified, not the image's natural size
+				wInch = ((static_cast<double>(imgProps.scaleX) / 100.0f) * imgProps.wGoal/ 1440.0f);
+				hInch = ((static_cast<double>(imgProps.scaleY) / 100.0f) * imgProps.hGoal/ 1440.0f);
+			} else {
+				wInch = ((static_cast<double>(imgProps.scaleX) / 100.0f) * (imgProps.width));
+				hInch = ((static_cast<double>(imgProps.scaleY) / 100.0f) * (imgProps.height));
+			}
 			break;
 		default:
 			resize = false;
@@ -347,7 +366,7 @@ bool IE_Imp_RTF::InsertImage (const UT_ByteBuf * buf, const char * image_name,
 
 		if (resize)
 		{
-			UT_LocaleTransactor(LC_NUMERIC, "C");
+			UT_LocaleTransactor t(LC_NUMERIC, "C");
 			UT_DEBUGMSG (("resizing...\n"));
 			UT_String_sprintf(propBuffer, "width:%fin; height:%fin",
 							  wInch, hInch);
@@ -419,6 +438,7 @@ bool IE_Imp_RTF::HandlePicture()
 			}
 			UT_DEBUGMSG(("Doing standard picture stuff with keyword %s \n",keyword));
 			keywordID = KeywordToID(reinterpret_cast<char *>(keyword));
+			UT_ASSERT(RTF_KW_cell  != keywordID);
 			switch (keywordID)
 			{
 			case RTF_KW_pngblip:
@@ -449,27 +469,35 @@ bool IE_Imp_RTF::HandlePicture()
 				}
 				break;
 			case RTF_KW_picscalex:
-				if ((imageProps.sizeType == RTFProps_ImageProps::ipstNone)
-					|| (imageProps.sizeType == RTFProps_ImageProps::ipstScale))
+				if ((parameterUsed) && (parameter != 100))       // scale the image if one of these two keywords appear
 				{
-					if ((parameterUsed) && (parameter != 100))
-					{
-						imageProps.sizeType = RTFProps_ImageProps::ipstScale;
-						imageProps.scaleX = static_cast<unsigned short>(parameter);
-					}
+					imageProps.sizeType = RTFProps_ImageProps::ipstScale;
+					imageProps.scaleX = static_cast<unsigned short>(parameter);
 				}
 				break;
 			case RTF_KW_picscaley:
-				if ((imageProps.sizeType == RTFProps_ImageProps::ipstNone)
-					|| (imageProps.sizeType == RTFProps_ImageProps::ipstScale))
+				if ((parameterUsed) && (parameter != 100))
 				{
-					if ((parameterUsed) && (parameter != 100))
-					{
-						imageProps.sizeType = RTFProps_ImageProps::ipstScale;
-						imageProps.scaleY = static_cast<unsigned short>(parameter);
-					}
+					imageProps.sizeType = RTFProps_ImageProps::ipstScale;
+					imageProps.scaleY = static_cast<unsigned short>(parameter);
 				}
 				break;
+			case RTF_KW_piccropt:
+				imageProps.cropt = parameter;
+				imageProps.bCrop = true;
+				break;
+			case RTF_KW_piccropb:
+				imageProps.cropb = parameter;
+				imageProps.bCrop = true;
+				break;
+			case RTF_KW_piccropl:
+				imageProps.cropl = parameter;
+				imageProps.bCrop = true;
+				break;
+			case RTF_KW_piccropr:
+				imageProps.cropr = parameter;
+				imageProps.bCrop = true;
+				break;
 			case RTF_KW_bin:
 				/* this code is completely broken. 
 				   if we encounter the \bin keyboard, then we must
@@ -661,7 +689,8 @@ void RTFProps_FrameProps::_setProperty(const PropertyPair *pair)
 //
 		UT_DEBUGMSG(("Found positioned Image \n"));
 	}
-	else {
+	else if( propName && propValue)
+	{
 		UT_DEBUGMSG(("unknown property %s with value %s\n", propName->utf8_str(),
 					 propValue->utf8_str() ));
 	}
@@ -816,7 +845,7 @@ bool IE_Imp_TextParaPropParser::tokenKeyword(IE_Imp_RTF * ie,
 										   RTF_KEYWORD_ID kwID, 
 										   UT_sint32 param, bool paramUsed)
 {
-	UT_DEBUGMSG(("IE_Imp_TextParPropParser::tokenKeyword()\n"));
+	xxx_UT_DEBUGMSG(("IE_Imp_TextParPropParser::tokenKeyword()\n"));
 	return ie->TranslateKeywordID(kwID, param, paramUsed);
 }
 
@@ -875,12 +904,14 @@ public:
 private:
 	IE_Imp_RTF * m_ieRTF;
 	RTFProps_FrameProps m_currentFrame;	
+	UT_sint32    m_iOrigTableDepth;
 };
 
 
 IE_Imp_ShpGroupParser::IE_Imp_ShpGroupParser(IE_Imp_RTF * ie) : m_ieRTF(ie) 
 {
 	m_currentFrame.clear();
+	m_iOrigTableDepth = m_ieRTF->getPasteDepth();
 }
 
 
@@ -893,7 +924,7 @@ IE_Imp_ShpGroupParser::~IE_Imp_ShpGroupParser()
 	//
 	// Close off any open tables
 	//
-	if(	m_ieRTF->getPasteDepth() > 0)
+	if(	(m_ieRTF->getPasteDepth() > 0) && (m_iOrigTableDepth < m_ieRTF->getPasteDepth()) )
 	{
 		m_ieRTF->closePastedTableIfNeeded();
 		if(m_ieRTF->bUseInsertNotAppend())
@@ -1074,7 +1105,7 @@ void IE_Imp_RTF::addFrame(RTFProps_FrameProps & frame)
 	UT_UTF8String_setProperty(sPropString,sP,sV); // fixme make other types
 
 	{
-		UT_LocaleTransactor(LC_NUMERIC, "C");
+		UT_LocaleTransactor t(LC_NUMERIC, "C");
 
 		//
 		// Shift positions a little for pasted frames so they don't
@@ -1083,7 +1114,8 @@ void IE_Imp_RTF::addFrame(RTFProps_FrameProps & frame)
 		double dOff = 0.0;
 		if(bUseInsertNotAppend())
 		{
-			dOff = 0.1; // 0.1 inches
+			dOff = 0.05; // 0.1 inches
+			dOff += 0.2*static_cast<double>(UT_rand())/static_cast<double>(UT_RAND_MAX);
 		}
 		double dV = dOff + static_cast<double>(frame.m_iLeftPos)/1440.0;
 		sV= UT_UTF8String_sprintf("%fin",dV);
@@ -1129,17 +1161,12 @@ void IE_Imp_RTF::addFrame(RTFProps_FrameProps & frame)
 	if(!bUseInsertNotAppend())
 	{
 		getDoc()->appendStrux(PTX_SectionFrame,attribs);
-		getDoc()->appendStrux(PTX_Block,NULL);
 	}
 	else
 	{
 		insertStrux(PTX_SectionFrame,attribs,NULL);
-		UT_DEBUGMSG((" Insert Block at Frame \n"));
-		markPasteBlock();
-		insertStrux(PTX_Block);
 	}
 	m_bFrameStruxIn = true;
-
 }
 
 /*!
@@ -1184,6 +1211,7 @@ void IE_Imp_RTF::HandleShapePict()
 		case RTF_TOKEN_KEYWORD:
 		{
 			keywordID = KeywordToID(reinterpret_cast<char *>(keyword));
+			UT_ASSERT(RTF_KW_cell  != keywordID);
 			switch (keywordID)
 			{
 			case RTF_KW_pict:
@@ -1247,6 +1275,7 @@ bool IE_Imp_RTF::HandleObject()
 		case RTF_TOKEN_KEYWORD:
 		{
 			keywordID = KeywordToID(reinterpret_cast<char *>(keyword));
+			UT_ASSERT(RTF_KW_cell  != keywordID);
 			switch (keywordID)
 			{
 			case RTF_KW_rsltrtf:
