@@ -1,19 +1,21 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -26,13 +28,15 @@
 #include "ut_mbtowc.h"
 #include "pd_Document.h"
 
+class pf_Frag_Strux;
+
 // Stream class can be File or Clipboard
 
-class ImportStream
+class ABI_EXPORT ImportStream
 {
  public:
 	ImportStream();
-	virtual ~ImportStream() {}
+	virtual ~ImportStream();
 	bool init(const char *szEncoding);
 	bool getChar(UT_UCSChar &b);
 	UT_UCSChar peekChar() { return m_ucsLookAhead; }
@@ -40,13 +44,13 @@ class ImportStream
 	virtual bool _getByte(unsigned char &b) = 0;
 	virtual bool getRawChar(UT_UCSChar &b);
 
-	bool _eof () const { return m_bEOF; }
-	void _eof (bool b) { m_bEOF = b; }
+	bool _get_eof () const { return m_bEOF; }
+	void _set_eof (bool b) { m_bEOF = b; }
 	UT_UCSChar _lookAhead () const { return m_ucsLookAhead; }
 	void _lookAhead ( UT_UCSChar c ) { m_ucsLookAhead = c; }
 
  private:
-	UT_Mbtowc m_Mbtowc;
+	UT_UCS4_mbtowc m_Mbtowc;
 	UT_UCSChar m_ucsLookAhead;
 	bool m_bEOF;
 	bool m_bRaw;
@@ -54,11 +58,11 @@ class ImportStream
 
 // File stream class
 
-class ImportStreamFile : public ImportStream
+class ABI_EXPORT ImportStreamFile : public ImportStream
 {
 public:
 	ImportStreamFile(FILE *pFile);
-	~ImportStreamFile() {}
+	~ImportStreamFile();
 	bool getChar();
 protected:
 	bool _getByte(unsigned char &b);
@@ -68,48 +72,37 @@ private:
 
 // Clipboard stream class
 
-class ImportStreamClipboard : public ImportStream
+class ABI_EXPORT ImportStreamClipboard : public ImportStream
 {
 public:
-	ImportStreamClipboard(unsigned char *pClipboard, UT_uint32 iLength);
-	~ImportStreamClipboard() {};
-	bool getChar();
+	ImportStreamClipboard(const unsigned char *pClipboard, UT_uint32 iLength);
+	~ImportStreamClipboard();
+	//	bool getChar();
 protected:
 	bool _getByte(unsigned char &b);
 private:
-	unsigned char *m_p;
-	unsigned char *m_pEnd;
-};
-
-// Helper class so we can parse files and clipboard with same code
-
-class Inserter
-{
-public:
-	Inserter(PD_Document * pDocument);
-	Inserter(PD_Document * pDocument, PT_DocPosition dPos);
-	bool insertBlock();
-	bool insertSpan(UT_GrowBuf &b);
-private:
-	PD_Document * m_pDocument;
-	bool m_bClipboard;
-	PT_DocPosition m_dPos;
+	const unsigned char *m_p;
+	const unsigned char *m_pEnd;
 };
 
 // The importer/reader for Plain Text Files.
 
-class IE_Imp_Text_Sniffer : public IE_ImpSniffer
+class ABI_EXPORT IE_Imp_Text_Sniffer : public IE_ImpSniffer
 {
 	friend class IE_Imp;
 	friend class IE_Imp_Text;
 
 public:
-	IE_Imp_Text_Sniffer() {}
-	virtual ~IE_Imp_Text_Sniffer() {}
+	IE_Imp_Text_Sniffer();
+	virtual ~IE_Imp_Text_Sniffer();
 
-	virtual bool recognizeContents (const char * szBuf,
+	UT_Confidence_t supportsMIME (const char * szMIME);
+
+	virtual UT_Confidence_t recognizeContents (const char * szBuf,
 									UT_uint32 iNumbytes);
-	virtual bool recognizeSuffix (const char * szSuffix);
+	const char * recognizeContentsType (const char * szBuf,
+									UT_uint32 iNumbytes);
+	virtual UT_Confidence_t recognizeSuffix (const char * szSuffix);
 	virtual bool getDlgLabels (const char ** szDesc,
 							   const char ** szSuffixList,
 							   IEFileType * ft);
@@ -128,18 +121,18 @@ protected:
 
 // The importer/reader for Plain Text Files with selectable encoding.
 
-class IE_Imp_EncodedText_Sniffer : public IE_ImpSniffer
+class ABI_EXPORT IE_Imp_EncodedText_Sniffer : public IE_ImpSniffer
 {
 	friend class IE_Imp;
 	friend class IE_Imp_Text;
 
 public:
-	IE_Imp_EncodedText_Sniffer() {}
-	virtual ~IE_Imp_EncodedText_Sniffer() {}
+	IE_Imp_EncodedText_Sniffer();
+	virtual ~IE_Imp_EncodedText_Sniffer();
 
-	virtual bool recognizeContents (const char * szBuf,
-									UT_uint32 iNumbytes);
-	virtual bool recognizeSuffix (const char * szSuffix);
+	virtual UT_Confidence_t recognizeContents (const char * szBuf,
+					    UT_uint32 iNumbytes);
+	virtual UT_Confidence_t recognizeSuffix (const char * szSuffix);
 	virtual bool getDlgLabels (const char ** szDesc,
 							   const char ** szSuffixList,
 							   IEFileType * ft);
@@ -149,31 +142,39 @@ public:
 protected:
 };
 
-class IE_Imp_Text : public IE_Imp
+class ABI_EXPORT IE_Imp_Text : public IE_Imp
 {
 public:
 	IE_Imp_Text(PD_Document * pDocument, bool bEncoded=false);
-	~IE_Imp_Text() {}
+	IE_Imp_Text(PD_Document * pDocument, const char * encoding);
+	virtual ~IE_Imp_Text();
 
 	virtual UT_Error	importFile(const char * szFilename);
-	virtual void		pasteFromBuffer(PD_DocumentRange * pDocRange,
-										unsigned char * pData, UT_uint32 lenData, const char * szEncoding = 0);
+	virtual bool		pasteFromBuffer(PD_DocumentRange * pDocRange,
+										const unsigned char * pData, UT_uint32 lenData, const char * szEncoding = 0);
 
 protected:
 	UT_Error			_recognizeEncoding(FILE * fp);
 	UT_Error			_recognizeEncoding(const char *szBuf, UT_uint32 iNumbytes);
 	virtual UT_Error	_constructStream(ImportStream *& pStream, FILE * fp);
 	UT_Error			_writeHeader(FILE * fp);
-	UT_Error			_parseStream(ImportStream * pStream, class Inserter & ins);
+	UT_Error			_parseStream(ImportStream * pStream);
 	bool				_doEncodingDialog(const char *szEncoding);
 	void				_setEncoding(const char *szEncoding);
 
+	bool _insertBlock ();
+	bool _insertSpan (UT_GrowBuf &b);
+
  private:
 	const char *	m_szEncoding;
+	bool m_bExplicitlySetEncoding;
 	bool			m_bIsEncoded;
 	bool			m_bIs16Bit;
 	bool			m_bUseBOM;
 	bool			m_bBigEndian;
+	bool            m_bBlockDirectionPending;
+	bool            m_bFirstBlockData;
+	pf_Frag_Strux * m_pBlock;
 };
 
 #endif /* IE_IMP_TEXT_H */
