@@ -607,9 +607,9 @@ void s_WML_Listener::_handleDataItems()
 				 * but I don't think so. */
 			}
 
-			if (!UT_strcmp(szMimeType, "image/svg-xml"))
+			if (!UT_strcmp(szMimeType, "image/svg+xml"))
 				UT_String_sprintf(fname, "%s/%s_%d.svg", fname.c_str(), szName, loc);
-			if (!UT_strcmp(szMimeType, "text/mathml"))
+			if (!UT_strcmp(szMimeType, "application/mathml+xml"))
 				UT_String_sprintf(fname, "%s/%s_%d.mathml", fname.c_str(), szName, loc);
 			else // PNG Image
 			{
