@@ -178,6 +178,7 @@ IE_Imp_WordPerfect::IE_Imp_WordPerfect(PD_Document * pDocument)
 	m_pCurrentListDefinition(NULL),
 	m_iCurrentListLevel(0),
 	m_bInCell(false),
+	m_iNumCoveredTableCellsInRow(0),
 	m_bHdrFtrOpenCount(0)
 {
 }
@@ -850,6 +851,7 @@ void IE_Imp_WordPerfect::openTableRow(const WPXPropertyList &propList)
 	}
 	
 	m_bInCell = false;
+	m_iNumCoveredTableCellsInRow = 0;
 }
 
 void IE_Imp_WordPerfect::openTableCell(const WPXPropertyList &propList)
@@ -872,7 +874,8 @@ void IE_Imp_WordPerfect::openTableCell(const WPXPropertyList &propList)
 	}
 	
 	UT_String propBuffer;
-	UT_String_sprintf(propBuffer, "left-attach:%d; right-attach:%d; top-attach:%d; bot-attach:%d", col, col+colSpan, row, row+rowSpan);
+	UT_String_sprintf(propBuffer, "left-attach:%d; right-attach:%d; top-attach:%d; bot-attach:%d",
+					col+m_iNumCoveredTableCellsInRow, col+m_iNumCoveredTableCellsInRow+colSpan, row, row+rowSpan);
 	
 	UT_String borderStyle;
 	// we only support bg-style:1 for now
@@ -913,6 +916,7 @@ void IE_Imp_WordPerfect::openTableCell(const WPXPropertyList &propList)
 	
 	X_CheckDocumentError(appendStrux(PTX_SectionCell, propsArray));
 	m_bInCell = true;
+	m_iNumCoveredTableCellsInRow += colSpan-1;
 }
 
 void IE_Imp_WordPerfect::closeTable()
